from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import sequencetools


def milliseconds_to_q_events(milliseconds, fuse_silences=False):
    '''Convert a list of millisecond durations to a list of
    ``QEvent`` instances.

    Negative duration values can be used to indicate silence.  Any resulting
    pitched ``QEvent`` instances will default to using middle-C.

    ::

        >>> durations = [100, -250, 500]
        >>> for x in quantizationtools.milliseconds_to_q_events(durations): x
        ...
        quantizationtools.PitchedQEvent(
            durationtools.Offset(0, 1),
            (NamedChromaticPitch("c'"),),
            attachments=()
            )
        quantizationtools.SilentQEvent(
            durationtools.Offset(100, 1),
            attachments=()
            )
        quantizationtools.PitchedQEvent(
            durationtools.Offset(350, 1),
            (NamedChromaticPitch("c'"),),
            attachments=()
            )
        quantizationtools.TerminalQEvent(
            durationtools.Offset(850, 1)
            )

    Return a list of ``QEvent`` instances.
    '''
    from abjad.tools import quantizationtools

    if fuse_silences:
        durations = [x for x in sequencetools.sum_consecutive_sequence_elements_by_sign(milliseconds, sign=[-1]) if x]
    else:
        durations = milliseconds
    offsets = mathtools.cumulative_sums_zero([abs(x) for x in durations])

    q_events = []
    for pair in zip(offsets, durations):
        offset = durationtools.Offset(pair[0])
        duration = pair[1]
        if duration < 0: # negative duration indicates silence
            q_event = quantizationtools.SilentQEvent(offset)
        else:
            q_event = quantizationtools.PitchedQEvent(offset, [0])
        q_events.append(q_event)

    q_events.append(quantizationtools.TerminalQEvent(durationtools.Offset(offsets[-1])))

    return q_events



from abjad.tools import durationtools
from abjad.tools.containertools.Container import Container


class FixedDurationContainer(Container):
    r'''.. versionadded:: 2.9

    Fixed-duration container::

        >>> container = containertools.FixedDurationContainer((3, 8), "c'8 d'8 e'8")

    ::

        >>> container
        FixedDurationContainer(Duration(3, 8), [Note("c'8"), Note("d'8"), Note("e'8")])

    ::

        >>> f(container)
        {
            c'8
            d'8
            e'8
        }


    Fixed-duration containers extend container behavior with format-time
    checking against a user-specified target duration.

    Return fixed-duration container.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_target_duration', )

    ### INITIALIZER ###

    def __init__(self, target_duration, music=None, **kwargs):
        Container.__init__(self, music=music, **kwargs)
        target_duration = durationtools.Duration(target_duration)
        assert 0 < target_duration
        self._target_duration = target_duration

    ### SPECIAL METHODS ###

    def __repr__(self):
        return '{}({!r}, {})'.format(self._class_name, self.target_duration, list(self[:]))

    ### PRIVATE METHODS ###

    def _check_duration(self):
        from abjad.tools import contexttools
        preprolatedg_duration = self.contents_duration
        if preprolatedg_duration < self.target_duration:
            raise UnderfullContainerError
        if self.target_duration < preprolatedg_duration:
            raise OverfullContainerError

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def is_full(self):
        '''True when preprolated duration equals target duration.
        '''
        return self.preprolated_duration == self.target_duration

    @property
    def is_misfilled(self):
        '''True when preprolated duration does not equal target duration.
        '''
        return not self.is_full

    @property
    def is_overfull(self):
        '''True when preprolated duration is greater than target duration.
        '''
        return self.target_duration < self.preprolated_duration

    @property
    def is_underfull(self):
        '''True when preprolated duration is less than target duration.
        '''
        return self.preprolated_duration < self.target_duration

    @property
    def lilypond_format(self):
        '''Read-only LilyPond format of fixed-duration container.
        '''
        self._check_duration()
        return self._format_component()

    ### READ / WRITE PUBLIC PROPERTIES ###

    @apply
    def target_duration():
        def fget(self):
            '''Read / write target duration of fixed-duration container.
            '''
            return self._target_duration
        def fset(self, target_duration):
            target_duration = durationtools.Duration(target_duration)
            assert 0 < target_duration
            self._target_duration = target_duration
        return property(**locals())



# -*- encoding: utf-8 -*-

from abjad import *
from abjad.demos.desordre.make_desordre_measure import make_desordre_measure


def make_desordre_staff(pitches):
    staff = Staff()

    for sequence in pitches:
        measure = make_desordre_measure(sequence)
        staff.append(measure)

    return staff



from abjad import *


def apply_rehearsal_marks(score):

    bell_voice = score['Bell Voice']

    measure_indices = [6, 12, 18, 24, 30, 36, 42, 48, 54, 60, 66, 72, 78, 84,
        90, 96, 102]

    for measure_index in measure_indices:
        marktools.LilyPondCommandMark(
            r'mark \default', 
            'before'
            )(bell_voice[measure_index])





def mask_intervals_with_intervals(masked_intervals, mask_intervals):
    '''Clip or remove all intervals in `masked_intervals` outside of the bounds
    defined in `mask_intervals`, while maintaining `masked_intervals`' payload contents ::

        >>> from abjad.tools.timeintervaltools import TimeInterval
        >>> from abjad.tools.timeintervaltools import TimeIntervalTree

    ::

        >>> a = TimeInterval(0, 10, {'a': 1})
        >>> b = TimeInterval(5, 15, {'b': 2})
        >>> tree = TimeIntervalTree([a, b])
        >>> mask = TimeInterval(4, 11)
        >>> timeintervaltools.mask_intervals_with_intervals(tree, mask)
        TimeIntervalTree([
            TimeInterval(Offset(4, 1), Offset(10, 1), {'a': 1}),
            TimeInterval(Offset(5, 1), Offset(11, 1), {'b': 2})
        ])

    Return TimeIntervalTree.
    '''
    from abjad.tools import timeintervaltools

    masked_tree = timeintervaltools.TimeIntervalTree(masked_intervals)
    mask_tree = timeintervaltools.TimeIntervalTree(mask_intervals)

    if not masked_tree or not mask_tree:
        return timeintervaltools.TimeIntervalTree([])

    start = min(mask_tree.start, masked_tree.start)
    stop = max(mask_tree.stop, masked_tree.stop)
    not_mask_tree = timeintervaltools.compute_logical_not_of_intervals_in_interval(
        mask_tree, timeintervaltools.TimeInterval(start, stop))

    if not not_mask_tree:
        return masked_tree

    not_mask_tree_bounds = timeintervaltools.get_all_unique_bounds_in_intervals(not_mask_tree)
    split_masked_tree = timeintervaltools.split_intervals_at_rationals(masked_tree, not_mask_tree_bounds)

    for interval in not_mask_tree:
        starts = set(split_masked_tree.find_intervals_starting_within_interval(interval))
        stops = set(split_masked_tree.find_intervals_stopping_within_interval(interval))
        intersection = set(starts).intersection(set(stops))
        split_masked_tree = timeintervaltools.TimeIntervalTree(
            list(set(split_masked_tree).difference(intersection)))

    return split_masked_tree



# TODO: limit output to string only
def _report_component_format_contributions(component, verbose=False, output='screen'):
    r'''Read-only string report of all format-time contributions
    made to `component` by all the different parts of the Abjad
    system plumbing.

    Set `verbose` to True or False.

    Set `output` to 'screen' or 'string'.
    '''
    from abjad.tools import componenttools
    from abjad.tools import spannertools
    from abjad.tools import leaftools

    if isinstance(component, leaftools.Leaf):
        return component._report_format_contributors()
    if isinstance(component, componenttools.Component):
        return component._formatter.report(verbose=verbose, output=output)
    elif isinstance(component, spanertools.Spanner):
        return component._format.report(output=output)
    else:
        raise TypeError('neither component nor spanner: "%s".' % component)



def remove_component_subtree_from_score_and_spanners(components):
    r'''.. versionadded:: 1.1

    Example 1. Remove one leaf from score::

        >>> voice = Voice("c'8 [ { d'8 e'8 } f'8 ]")
        >>> spannertools.GlissandoSpanner(voice.leaves)
        GlissandoSpanner(c'8, d'8, e'8, f'8)

    ::

        >>> f(voice)
        \new Voice {
            c'8 [ \glissando
            {
                d'8 \glissando
                e'8 \glissando
            }
            f'8 ]
        }

    ::

        >>> componenttools.remove_component_subtree_from_score_and_spanners(voice.leaves[1:2])
        (Note("d'8"),)

    ::

        >>> f(voice)
        \new Voice {
            c'8 [ \glissando
            {
                e'8 \glissando
            }
            f'8 ]
        }

    Example 2. Remove contiguous leaves from score::

        >>> voice = Voice("c'8 [ { d'8 e'8 } f'8 ]")
        >>> spannertools.GlissandoSpanner(voice.leaves)
        GlissandoSpanner(c'8, d'8, e'8, f'8)

    ::

        >>> f(voice)
        \new Voice {
            c'8 [ \glissando
            {
                d'8 \glissando
                e'8 \glissando
            }
            f'8 ]
        }

    ::

        >>> componenttools.remove_component_subtree_from_score_and_spanners(voice.leaves[:2])
        (Note("c'8"), Note("d'8"))

    ::

        >>> f(voice)
        \new Voice {
            {
                e'8 [ \glissando
            }
            f'8 ]
        }

    Example 3. Remove noncontiguous leaves from score::

        >>> voice = Voice("c'8 [ { d'8 e'8 } f'8 ]")
        >>> spannertools.GlissandoSpanner(voice.leaves)
        GlissandoSpanner(c'8, d'8, e'8, f'8)

    ::

        >>> f(voice)
        \new Voice {
            c'8 [ \glissando
            {
                d'8 \glissando
                e'8 \glissando
            }
            f'8 ]
        }

    ::

        >>> componenttools.remove_component_subtree_from_score_and_spanners(
        ... [voice.leaves[0], voice.leaves[2]])
        [Note("c'8"), Note("e'8")]

    ::

        >>> f(voice)
        \new Voice {
            { d'8 [ \glissando
            }
            f'8 ]
        }

    Example 4. Remove container from score::

        >>> voice = Voice("c'8 [ { d'8 e'8 } f'8 ]")
        >>> spannertools.GlissandoSpanner(voice.leaves)
        GlissandoSpanner(c'8, d'8, e'8, f'8)

    ::

        >>> f(voice)
        \new Voice {
            c'8 [ \glissando
            {
                d'8 \glissando
                e'8 \glissando
            }
            f'8 ]
        }

    ::

        >>> componenttools.remove_component_subtree_from_score_and_spanners(voice[1:2])
        Selection({d'8, e'8},)

    ::

        >>> f(voice)
        \new Voice {
            c'8 [ \glissando
            f'8 ]
        }

    Withdraw `components` and children of `components` from spanners.

    Return either tuple or list of `components` and children of `components`.

    .. todo:: regularize return value of function.

    .. versionchanged:: 2.0
        renamed ``componenttools.detach()`` to
        ``componenttools.remove_component_subtree_from_score_and_spanners()``.
    '''
    from abjad.tools import componenttools
    from abjad.tools.spannertools._withdraw_components_in_expr_from_attached_spanners import \
        _withdraw_components_in_expr_from_attached_spanners

    assert componenttools.all_are_components(components)

    for component in components:
        component._cut()
        _withdraw_components_in_expr_from_attached_spanners([component])
    return components



from abjad import *


def test_Parentage_is_orphan_01():

    staff = Staff("c'8 d'8 e'8 f'8")

    assert staff.parentage.is_orphan
    for note in staff:
        assert not note.parentage.is_orphan



import abc
from abjad.tools.instrumenttools._ReedInstrument import _ReedInstrument


class _SingleReedInstrument(_ReedInstrument):
    '''.. versionadded:: 2.0

    Abjad model of single-reed instruments.
    '''
    __metaclass__ = abc.ABCMeta

    def __init__(self, **kwargs):
        _ReedInstrument.__init__(self, **kwargs)
        self._default_instrument_name = 'single reed instrument'
        self._default_performer_names.append('single reed player')



from abjad import *


def test_RhythmTreeContainer_contents_duration_01(): 

    leaf_a = rhythmtreetools.RhythmTreeLeaf(duration=3)
    leaf_b = rhythmtreetools.RhythmTreeLeaf(duration=3)
    leaf_c = rhythmtreetools.RhythmTreeLeaf(duration=2)
    subcontainer = rhythmtreetools.RhythmTreeContainer(duration=2, children=[leaf_b, leaf_c])
    leaf_d = rhythmtreetools.RhythmTreeLeaf(duration=1)

    container = rhythmtreetools.RhythmTreeContainer(duration=1, children=[leaf_a, subcontainer, leaf_d])
    
    assert container.contents_duration == 6
    assert subcontainer.contents_duration == 5



from abjad import *


def test_componenttools_split_components_at_offsets_01():
    '''Cyclically split note in score. Don't fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    beamtools.apply_beam_spanners_to_measures_in_expr(staff)
    spannertools.SlurSpanner(staff.leaves)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    parts = componenttools.split_components_at_offsets(
        staff[0][1:2], [(3, 64)], cyclic=True, fracture_spanners=False)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'32. ~
            d'32. ~
            d'32 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 3
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 2/8\n\t\tc'8 [ (\n\t\td'32. ~\n\t\td'32. ~\n\t\td'32 ]\n\t}\n\t{\n\t\te'8 [\n\t\tf'8 ] )\n\t}\n}"


def test_componenttools_split_components_at_offsets_02():
    '''Cyclically split consecutive notes in score. Don't fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    beamtools.apply_beam_spanners_to_measures_in_expr(staff)
    spannertools.SlurSpanner(staff.leaves)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    parts = componenttools.split_components_at_offsets(
        staff.leaves, [(3, 32)], cyclic=True, fracture_spanners=False)

    r'''
    \new Staff {
        {
            \time 2/8
            c'16. [ ( ~
            c'32
            d'16. ~
            d'32 ]
        }
        {
            e'16. [ ~
            e'32
            f'8 ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 6
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 2/8\n\t\tc'16. [ ( ~\n\t\tc'32\n\t\td'16 ~\n\t\td'16 ]\n\t}\n\t{\n\t\te'32 [ ~\n\t\te'16.\n\t\tf'16. ~\n\t\tf'32 ] )\n\t}\n}"


def test_componenttools_split_components_at_offsets_03():
    '''Cyclically split measure in score. Don't fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    beamtools.apply_beam_spanners_to_measures_in_expr(staff)
    spannertools.SlurSpanner(staff.leaves)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    parts = componenttools.split_components_at_offsets(staff[:1], [(3, 32)], 
        cyclic=True, fracture_spanners=False, tie_split_notes=False)

    r'''
    \new Staff {
        {
            \time 3/32
            c'16. [ (
        }
        {
            c'32
            d'16
        }
        {
            \time 2/32
            d'16 ]
        }
        {
            \time 2/8
            e'8 [
            f'8 ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 3
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 3/32\n\t\tc'16. [ (\n\t}\n\t{\n\t\tc'32\n\t\td'16\n\t}\n\t{\n\t\t\\time 2/32\n\t\td'16 ]\n\t}\n\t{\n\t\t\\time 2/8\n\t\te'8 [\n\t\tf'8 ] )\n\t}\n}"


def test_componenttools_split_components_at_offsets_04():
    '''Cyclically split consecutive measures in score. Don't fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    beamtools.apply_beam_spanners_to_measures_in_expr(staff)
    spannertools.SlurSpanner(staff.leaves)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    parts = componenttools.split_components_at_offsets(staff[:], [(3, 32)], 
        cyclic=True, fracture_spanners=False, tie_split_notes=False)

    r'''
    \new Staff {
        {
            \time 3/32
            c'16. [ (
        }
        {
            c'32
            d'16
        }
        {
            \time 2/32
            d'16 ]
        }
        {
            \time 1/32
            e'32 [
        }
        {
            \time 3/32
            e'16.
        }
        {
            f'16.
        }
        {
            \time 1/32
            f'32 ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 6
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 3/32\n\t\tc'16. [ (\n\t}\n\t{\n\t\tc'32\n\t\td'16\n\t}\n\t{\n\t\t\\time 2/32\n\t\td'16 ]\n\t}\n\t{\n\t\t\\time 1/32\n\t\te'32 [\n\t}\n\t{\n\t\t\\time 3/32\n\t\te'16.\n\t}\n\t{\n\t\tf'16.\n\t}\n\t{\n\t\t\\time 1/32\n\t\tf'32 ] )\n\t}\n}"


def test_componenttools_split_components_at_offsets_05():
    '''Cyclically split orphan measures. Don't fracture spanners.
    '''

    measures = [Measure((2, 8), "c'8 d'8"), Measure((2, 8), "e'8 f'8")]
    beamtools.apply_beam_spanners_to_measures_in_expr(measures)

    parts = componenttools.split_components_at_offsets(measures, [(3, 32)], 
        cyclic=True, fracture_spanners=False, tie_split_notes=False)

    music = sequencetools.flatten_sequence(parts)
    staff = Staff(music)

    r'''
    \new Staff {
        {
            \time 3/32
            c'16. [
        }
        {
            c'32
            d'16
        }
        {
            \time 2/32
            d'16 ]
        }
        {
            \time 1/32
            e'32 [
        }
        {
            \time 3/32
            e'16.
        }
        {
            f'16.
        }
        {
            \time 1/32
            f'32 ]
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 6
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 3/32\n\t\tc'16. [\n\t}\n\t{\n\t\tc'32\n\t\td'16\n\t}\n\t{\n\t\t\\time 2/32\n\t\td'16 ]\n\t}\n\t{\n\t\t\\time 1/32\n\t\te'32 [\n\t}\n\t{\n\t\t\\time 3/32\n\t\te'16.\n\t}\n\t{\n\t\tf'16.\n\t}\n\t{\n\t\t\\time 1/32\n\t\tf'32 ]\n\t}\n}"


def test_componenttools_split_components_at_offsets_06():
    '''Cyclically split note in score. Don't fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    beamtools.apply_beam_spanners_to_measures_in_expr(staff)
    spannertools.SlurSpanner(staff.leaves)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    parts = componenttools.split_components_at_offsets(staff[0][1:], [(1, 32)], 
        cyclic=True, fracture_spanners=False, tie_split_notes=True)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'32 ~
            d'32 ~
            d'32 ~
            d'32 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 4
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 2/8\n\t\tc'8 [ (\n\t\td'32 ~\n\t\td'32 ~\n\t\td'32 ~\n\t\td'32 ]\n\t}\n\t{\n\t\te'8 [\n\t\tf'8 ] )\n\t}\n}"


def test_componenttools_split_components_at_offsets_07():
    '''Cyclically split consecutive notes in score. Don't fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    beamtools.apply_beam_spanners_to_measures_in_expr(staff)
    spannertools.SlurSpanner(staff.leaves)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    parts = componenttools.split_components_at_offsets(staff.leaves, [(1, 16)], 
        cyclic=True, fracture_spanners=False, tie_split_notes=True)

    r'''
    \new Staff {
        {
            \time 2/8
            c'16 [ ( ~
            c'16
            d'16 ~
            d'16 ]
        }
        {
            e'16 [ ~
            e'16
            f'16 ~
            f'16 ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 8
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 2/8\n\t\tc'16 [ ( ~\n\t\tc'16\n\t\td'16 ~\n\t\td'16 ]\n\t}\n\t{\n\t\te'16 [ ~\n\t\te'16\n\t\tf'16 ~\n\t\tf'16 ] )\n\t}\n}"


def test_componenttools_split_components_at_offsets_08():
    '''Cyclically split measure in score. Don't fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    beamtools.apply_beam_spanners_to_measures_in_expr(staff)
    spannertools.SlurSpanner(staff.leaves)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    parts = componenttools.split_components_at_offsets(staff[:1], [(1, 16)], 
        cyclic=True, fracture_spanners=False, tie_split_notes=True)

    r'''
    \new Staff {
        {
            \time 1/16
            c'16 [ ( ~
        }
        {
            c'16
        }
        {
            d'16 ~
        }
        {
            d'16 ]
        }
        {
            \time 2/8
            e'8 [
            f'8 ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 4
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 1/16\n\t\tc'16 [ ( ~\n\t}\n\t{\n\t\tc'16\n\t}\n\t{\n\t\td'16 ~\n\t}\n\t{\n\t\td'16 ]\n\t}\n\t{\n\t\t\\time 2/8\n\t\te'8 [\n\t\tf'8 ] )\n\t}\n}"


def test_componenttools_split_components_at_offsets_09():
    '''Cyclically split consecutive measures in score. Don't fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    beamtools.apply_beam_spanners_to_measures_in_expr(staff)
    spannertools.SlurSpanner(staff.leaves)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    parts = componenttools.split_components_at_offsets(staff[:], [(3, 32)], 
        cyclic=True, fracture_spanners=False, tie_split_notes=True)

    r'''
    \new Staff {
        {
            \time 3/32
            c'16. [ ( ~
        }
        {
            c'32
            d'16 ~
        }
        {
            \time 2/32
            d'16 ]
        }
        {
            \time 1/32
            e'32 [ ~
        }
        {
            \time 3/32
            e'16.
        }
        {
            f'16. ~
        }
        {
            \time 1/32
            f'32 ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 6
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 3/32\n\t\tc'16. [ ( ~\n\t}\n\t{\n\t\tc'32\n\t\td'16 ~\n\t}\n\t{\n\t\t\\time 2/32\n\t\td'16 ]\n\t}\n\t{\n\t\t\\time 1/32\n\t\te'32 [ ~\n\t}\n\t{\n\t\t\\time 3/32\n\t\te'16.\n\t}\n\t{\n\t\tf'16. ~\n\t}\n\t{\n\t\t\\time 1/32\n\t\tf'32 ] )\n\t}\n}"


def test_componenttools_split_components_at_offsets_10():
    '''Cyclically split note in score. Fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    beamtools.apply_beam_spanners_to_measures_in_expr(staff)
    spannertools.SlurSpanner(staff.leaves)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    parts = componenttools.split_components_at_offsets(
        staff[0][1:2], [(3, 64)], cyclic=True, fracture_spanners=True)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'32. )
            d'32. ( )
            d'64 ( ~
            d'64 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 3
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 2/8\n\t\tc'8 [ (\n\t\td'32. ) ~\n\t\td'32. ( ) ~\n\t\td'32 ] (\n\t}\n\t{\n\t\te'8 [\n\t\tf'8 ] )\n\t}\n}"


def test_componenttools_split_components_at_offsets_11():
    '''Cyclically split consecutive notes in score. Fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    beamtools.apply_beam_spanners_to_measures_in_expr(staff)
    spannertools.SlurSpanner(staff.leaves)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    parts = componenttools.split_components_at_offsets(
        staff.leaves, [(3, 32)], cyclic=True, fracture_spanners=True)

    r'''
    \new Staff {
        {
            \time 2/8
            c'16. ( ) [
            c'32 (
            d'16 )
            d'16 ] (
        }
        {
            e'32 ) [
            e'16. (
            f'16. )
            f'32 ] ( )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 6
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 2/8\n\t\tc'16. ( ) [ ~\n\t\tc'32 (\n\t\td'16 ) ~\n\t\td'16 ] (\n\t}\n\t{\n\t\te'32 ) [ ~\n\t\te'16. (\n\t\tf'16. ) ~\n\t\tf'32 ] ( )\n\t}\n}"


def test_componenttools_split_components_at_offsets_12():
    '''Cyclically split measure in score. Fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    beamtools.apply_beam_spanners_to_measures_in_expr(staff)
    spannertools.SlurSpanner(staff.leaves)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    parts = componenttools.split_components_at_offsets(staff[:1], [(3, 32)], 
        cyclic=True, fracture_spanners=True, tie_split_notes=False)

    r'''
    \new Staff {
        {
            \time 3/32
            c'16. [ ] ( )
        }
        {
            c'32 [ (
            d'16 ] )
        }
        {
            \time 2/32
            d'16 [ ] (
        }
        {
            \time 2/8
            e'8 [
            f'8 ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 3
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 3/32\n\t\tc'16. [ ] ( )\n\t}\n\t{\n\t\tc'32 [ (\n\t\td'16 ] )\n\t}\n\t{\n\t\t\\time 2/32\n\t\td'16 [ ] (\n\t}\n\t{\n\t\t\\time 2/8\n\t\te'8 [\n\t\tf'8 ] )\n\t}\n}"


def test_componenttools_split_components_at_offsets_13():
    '''Cyclically split consecutive measures in score. Fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    beamtools.apply_beam_spanners_to_measures_in_expr(staff)
    spannertools.SlurSpanner(staff.leaves)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    parts = componenttools.split_components_at_offsets(staff[:], [(3, 32)], 
        cyclic=True, fracture_spanners=True, tie_split_notes=False)

    r'''
    \new Staff {
        {
            \time 3/32
            c'16. [ ] ( )
        }
        {
            c'32 [ (
            d'16 ] )
        }
        {
            \time 2/32
            d'16 [ ] (
        }
        {
            \time 1/32
            e'32 [ ] )
        }
        {
            \time 3/32
            e'16. [ ] ( )
        }
        {
            f'16. [ ] ( )
        }
        {
            \time 1/32
            f'32 [ ] ( )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 6
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 3/32\n\t\tc'16. [ ] ( )\n\t}\n\t{\n\t\tc'32 [ (\n\t\td'16 ] )\n\t}\n\t{\n\t\t\\time 2/32\n\t\td'16 [ ] (\n\t}\n\t{\n\t\t\\time 1/32\n\t\te'32 [ ] )\n\t}\n\t{\n\t\t\\time 3/32\n\t\te'16. [ ] ( )\n\t}\n\t{\n\t\tf'16. [ ] ( )\n\t}\n\t{\n\t\t\\time 1/32\n\t\tf'32 [ ] ( )\n\t}\n}"


def test_componenttools_split_components_at_offsets_14():
    '''Cyclically split orphan notes.
    '''

    notes = [Note("c'8"), Note("d'8"), Note("e'8"), Note("f'8")]

    parts = componenttools.split_components_at_offsets(
        notes, [(3, 32)], cyclic=True, fracture_spanners=True)

    music = sequencetools.flatten_sequence(parts)
    staff = Staff(music)

    r'''
    \new Staff {
        c'16. ~
        c'32
        d'16 ~
        d'16
        e'32 ~
        e'16.
        f'16. ~
        f'32
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 6
    assert staff.lilypond_format == "\\new Staff {\n\tc'16. ~\n\tc'32\n\td'16 ~\n\td'16\n\te'32 ~\n\te'16.\n\tf'16. ~\n\tf'32\n}"


def test_componenttools_split_components_at_offsets_15():
    '''Cyclically split orphan measures. Fracture spanners.
    '''

    measures = [Measure((2, 8), "c'8 d'8"), Measure((2, 8), "e'8 f'8")]
    beamtools.apply_beam_spanners_to_measures_in_expr(measures)

    parts = componenttools.split_components_at_offsets(measures, [(3, 32)], 
        cyclic=True, fracture_spanners=True, tie_split_notes=False)

    music = sequencetools.flatten_sequence(parts)
    staff = Staff(music)

    r'''
    \new Staff {
        {
            \time 3/32
            c'16. [ ]
        }
        {
            c'32 [
            d'16 ]
        }
        {
            \time 2/32
            d'16 [ ]
        }
        {
            \time 1/32
            e'32 [ ]
        }
        {
            \time 3/32
            e'16. [ ]
        }
        {
            f'16. [ ]
        }
        {
            \time 1/32
            f'32 [ ]
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 6
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 3/32\n\t\tc'16. [ ]\n\t}\n\t{\n\t\tc'32 [\n\t\td'16 ]\n\t}\n\t{\n\t\t\\time 2/32\n\t\td'16 [ ]\n\t}\n\t{\n\t\t\\time 1/32\n\t\te'32 [ ]\n\t}\n\t{\n\t\t\\time 3/32\n\t\te'16. [ ]\n\t}\n\t{\n\t\tf'16. [ ]\n\t}\n\t{\n\t\t\\time 1/32\n\t\tf'32 [ ]\n\t}\n}"


def test_componenttools_split_components_at_offsets_16():
    '''Cyclically split note in score. Fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    beamtools.apply_beam_spanners_to_measures_in_expr(staff)
    spannertools.SlurSpanner(staff.leaves)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    parts = componenttools.split_components_at_offsets(staff[0][1:], [(1, 32)], 
        cyclic=True, fracture_spanners=True, tie_split_notes=True)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'32 ) ~
            d'32 ( ) ~
            d'32 ( ) ~
            d'32 ] (
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 4
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 2/8\n\t\tc'8 [ (\n\t\td'32 ) ~\n\t\td'32 ( ) ~\n\t\td'32 ( ) ~\n\t\td'32 ] (\n\t}\n\t{\n\t\te'8 [\n\t\tf'8 ] )\n\t}\n}"


def test_componenttools_split_components_at_offsets_17():
    '''Cyclically split consecutive notes in score. Fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    beamtools.apply_beam_spanners_to_measures_in_expr(staff)
    spannertools.SlurSpanner(staff.leaves)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    parts = componenttools.split_components_at_offsets(staff.leaves, [(1, 16)], 
        cyclic=True, fracture_spanners=True, tie_split_notes=True)

    r'''
    \new Staff {
        {
            \time 2/8
            c'16 ( ) [ ~
            c'16 (
            d'16 ) ~
            d'16 ] (
        }
        {
            e'16 ) [ ~
            e'16 (
            f'16 ) ~
            f'16 ] ( )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 8
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 2/8\n\t\tc'16 ( ) [ ~\n\t\tc'16 (\n\t\td'16 ) ~\n\t\td'16 ] (\n\t}\n\t{\n\t\te'16 ) [ ~\n\t\te'16 (\n\t\tf'16 ) ~\n\t\tf'16 ] ( )\n\t}\n}"


def test_componenttools_split_components_at_offsets_18():
    '''Cyclically split measure in score. Fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    beamtools.apply_beam_spanners_to_measures_in_expr(staff)
    spannertools.SlurSpanner(staff.leaves)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    parts = componenttools.split_components_at_offsets(staff[:1], [(1, 16)], 
        cyclic=True, fracture_spanners=True, tie_split_notes=True)

    r'''
    \new Staff {
        {
            \time 1/16
            c'16 [ ] ( ) ~
        }
        {
            c'16 [ ] ( )
        }
        {
            d'16 [ ] ( ) ~
        }
        {
            d'16 [ ] (
        }
        {
            \time 2/8
            e'8 [
            f'8 ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 4
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 1/16\n\t\tc'16 [ ] ( ) ~\n\t}\n\t{\n\t\tc'16 [ ] ( )\n\t}\n\t{\n\t\td'16 [ ] ( ) ~\n\t}\n\t{\n\t\td'16 [ ] (\n\t}\n\t{\n\t\t\\time 2/8\n\t\te'8 [\n\t\tf'8 ] )\n\t}\n}"


def test_componenttools_split_components_at_offsets_19():
    '''Cyclically split consecutive measures in score. Fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    beamtools.apply_beam_spanners_to_measures_in_expr(staff)
    spannertools.SlurSpanner(staff.leaves)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    parts = componenttools.split_components_at_offsets(staff[:], [(3, 32)], 
        cyclic=True, fracture_spanners=True, tie_split_notes=True)

    r'''
    \new Staff {
        {
            \time 3/32
            c'16. [ ] ( ) ~
        }
        {
            c'32 [ (
            d'16 ] ) ~
        }
        {
            \time 2/32
            d'16 [ ] (
        }
        {
            \time 1/32
            e'32 [ ] ) ~
        }
        {
            \time 3/32
            e'16. [ ] ( )
        }
        {
            f'16. [ ] ( ) ~
        }
        {
            \time 1/32
            f'32 [ ] ( )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 6
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 3/32\n\t\tc'16. [ ] ( ) ~\n\t}\n\t{\n\t\tc'32 [ (\n\t\td'16 ] ) ~\n\t}\n\t{\n\t\t\\time 2/32\n\t\td'16 [ ] (\n\t}\n\t{\n\t\t\\time 1/32\n\t\te'32 [ ] ) ~\n\t}\n\t{\n\t\t\\time 3/32\n\t\te'16. [ ] ( )\n\t}\n\t{\n\t\tf'16. [ ] ( ) ~\n\t}\n\t{\n\t\t\\time 1/32\n\t\tf'32 [ ] ( )\n\t}\n}"


def test_componenttools_split_components_at_offsets_20():
    '''Force split measure in score. Do not fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    beamtools.apply_beam_spanners_to_measures_in_expr(staff)
    spannertools.SlurSpanner(staff.leaves)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    parts = componenttools.split_components_at_offsets(
        staff[:1], [(1, 32), (3, 32), (5, 32)], 
        cyclic=False, fracture_spanners=False, tie_split_notes=False)

    r'''
    \new Staff {
        {
            \time 1/32
            c'32 [ (
        }
        {
            \time 3/32
            c'16.
        }
        {
            \time 4/32
            d'8 ]
        }
        {
            \time 2/8
            e'8 [
            f'8 ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 3
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 1/32\n\t\tc'32 [ (\n\t}\n\t{\n\t\t\\time 3/32\n\t\tc'16.\n\t}\n\t{\n\t\t\\time 4/32\n\t\td'8 ]\n\t}\n\t{\n\t\t\\time 2/8\n\t\te'8 [\n\t\tf'8 ] )\n\t}\n}"


def test_componenttools_split_components_at_offsets_21():
    '''Force split consecutive measures in score. Do not fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    beamtools.apply_beam_spanners_to_measures_in_expr(staff)
    spannertools.SlurSpanner(staff.leaves)

    r'''
    \new Staff {
        {
            c'8 [ (
            d'8 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    parts = componenttools.split_components_at_offsets(
        staff[:], [(1, 32), (3, 32), (5, 32)], 
        cyclic=False, fracture_spanners=False, tie_split_notes=False)

    r'''
    \new Staff {
        {
            \time 1/32
            c'32 [ (
        }
        {
            \time 3/32
            c'16.
        }
        {
            \time 4/32
            d'8 ]
        }
        {
            \time 1/32
            e'32 [
        }
        {
            \time 7/32
            e'16.
            f'8 ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 4
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 1/32\n\t\tc'32 [ (\n\t}\n\t{\n\t\t\\time 3/32\n\t\tc'16.\n\t}\n\t{\n\t\t\\time 4/32\n\t\td'8 ]\n\t}\n\t{\n\t\t\\time 1/32\n\t\te'32 [\n\t}\n\t{\n\t\t\\time 7/32\n\t\te'16.\n\t\tf'8 ] )\n\t}\n}"


def test_componenttools_split_components_at_offsets_22():
    '''Force split measure in score. Fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    beamtools.apply_beam_spanners_to_measures_in_expr(staff)
    spannertools.SlurSpanner(staff.leaves)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    parts = componenttools.split_components_at_offsets(
        staff[:1], [(1, 32), (3, 32), (5, 32)], 
        cyclic=False, fracture_spanners=True, tie_split_notes=False)

    r'''
    \new Staff {
        {
            \time 1/32
            c'32 [ ] ( )
        }
        {
            \time 3/32
            c'16. [ ] ( )
        }
        {
            \time 4/32
            d'8 [ ] (
        }
        {
            \time 2/8
            e'8 [
            f'8 ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 3
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 1/32\n\t\tc'32 [ ] ( )\n\t}\n\t{\n\t\t\\time 3/32\n\t\tc'16. [ ] ( )\n\t}\n\t{\n\t\t\\time 4/32\n\t\td'8 [ ] (\n\t}\n\t{\n\t\t\\time 2/8\n\t\te'8 [\n\t\tf'8 ] )\n\t}\n}"


def test_componenttools_split_components_at_offsets_23():
    '''Force split consecutive measures in score. Fracture spanners.
    '''

    staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")
    beamtools.apply_beam_spanners_to_measures_in_expr(staff)
    spannertools.SlurSpanner(staff.leaves)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            e'8 [
            f'8 ] )
        }
    }
    '''

    parts = componenttools.split_components_at_offsets(
        staff[:], [(1, 32), (3, 32), (5, 32)], 
        cyclic=False, fracture_spanners=True, tie_split_notes=False)

    r'''
    \new Staff {
        {
            \time 1/32
            c'32 [ ] ( )
        }
        {
            \time 3/32
            c'16. [ ] ( )
        }
        {
            \time 4/32
            d'8 [ ] (
        }
        {
            \time 1/32
            e'32 [ ] )
        }
        {
            \time 7/32
            e'16. [ (
            f'8 ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 4
    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 1/32\n\t\tc'32 [ ] ( )\n\t}\n\t{\n\t\t\\time 3/32\n\t\tc'16. [ ] ( )\n\t}\n\t{\n\t\t\\time 4/32\n\t\td'8 [ ] (\n\t}\n\t{\n\t\t\\time 1/32\n\t\te'32 [ ] )\n\t}\n\t{\n\t\t\\time 7/32\n\t\te'16. [ (\n\t\tf'8 ] )\n\t}\n}"


def test_componenttools_split_components_at_offsets_24():
    '''Force split orphan note. Offsets sum to less than note duration.
    '''

    note = Note("c'4")

    parts = componenttools.split_components_at_offsets(
        [note], [(1, 32), (5, 32)], 
        cyclic=False, fracture_spanners=True, tie_split_notes=False)

    notes = sequencetools.flatten_sequence(parts)
    staff = Staff(notes)

    r'''
    \new Staff {
        c'32 ~
        c'8 ~
        c'32 ~
        c'16
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert len(parts) == 3
    assert staff.lilypond_format == "\\new Staff {\n\tc'32 ~\n\tc'8 ~\n\tc'32 ~\n\tc'16\n}"


def test_componenttools_split_components_at_offsets_25():
    '''Force split note in score. Fracture spanners.
    '''

    staff = Staff("c'8 [ ]")

    r'''
    \new Staff {
        c'8 [ ]
    }
    '''

    parts = componenttools.split_components_at_offsets(
        staff[:], [(1, 64), (5, 64)], 
        cyclic=False, fracture_spanners=True, tie_split_notes=False)

    r'''
    \new Staff {
        c'64 [ ]
        c'16 [ ~
        c'64 ] ~
        c'32 [ ]
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert staff.lilypond_format == "\\new Staff {\n\tc'64 [ ] ~\n\tc'16 [ ~\n\tc'64 ] ~\n\tc'32 [ ]\n}"



def get_annotation_attached_to_component(component):
    r'''.. versionadded:: 2.0

    Get exactly one annotation attached to `component`::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> marktools.Annotation('special information')(staff[0])
        Annotation('special information')(c'8)

    ::

        >>> f(staff)
        \new Staff {
            c'8
            d'8
            e'8
            f'8
        }

    ::

        >>> marktools.get_annotation_attached_to_component(staff[0])
        Annotation('special information')(c'8)

    Return one annotation.

    Raise missing mark error when no annotation is attached.

    Raise extra mark error when more than one annotation is attached.
    '''
    from abjad.tools import marktools

    annotations = marktools.get_annotations_attached_to_component(component)
    if not annotations:
        raise MissingMarkError
    elif 1 < len(annotations):
        raise ExtraMarkError
    else:
        return annotations[0]



import copy
from abjad.tools import durationtools
from abjad.tools import marktools
from abjad.tools import notetools
from abjad.tools import tietools


def edit_second_violin_voice(score, durated_reservoir):

    voice = score['Second Violin Voice']
    descents = durated_reservoir['Second Violin']

    copied_descent = list(copy.deepcopy(descents[-1]))
    copied_descent[-1].written_duration = durationtools.Duration(1, 1)
    copied_descent.append(notetools.Note('a2'))
    for leaf in copied_descent:
        marktools.Articulation('accent')(leaf)
        marktools.Articulation('tenuto')(leaf)
    voice.extend(copied_descent)

    final_sustain = []
    for _ in range(32):
        final_sustain.append(notetools.Note('a1.'))
    final_sustain.append(notetools.Note('a2'))
    marktools.Articulation('accent')(final_sustain[0])
    marktools.Articulation('tenuto')(final_sustain[0])

    voice.extend(final_sustain)
    tietools.TieSpanner(final_sustain)
    voice.extend('r4 r2.')




from abjad.tools import durationtools
from abjad.tools import leaftools
from abjad.tools import mathtools
from abjad.tools import notetools
from abjad.tools import resttools
from abjad.tools import sequencetools


def make_tuplet_from_duration_and_ratio(duration, proportions, 
    avoid_dots=True, decrease_durations_monotonically=True, is_diminution=True):
    r'''.. versionadded:: 2.10

    Make tuplet from `duration` and `proportions`.

    Example set 1. Make augmented tuplet from `duration` and `proportions` and
    avoid dots.

    Return tupletted leaves strictly without dots when all `proportions` equal
    ``1``::

        >>> print tuplettools.make_tuplet_from_duration_and_ratio(
        ...     Duration(3, 16), [1, 1, 1, -1, -1], avoid_dots=True,
        ...     is_diminution=False)
        {@ 5:6 c'32, c'32, c'32, r32, r32 @}

    Allow tupletted leaves to return with dots when some `proportions` do not
    equal ``1``::

        >>> print tuplettools.make_tuplet_from_duration_and_ratio(
        ...     Duration(3, 16), [1, -2, -2, 3, 3], avoid_dots=True,
        ...     is_diminution=False)
        {@ 11:12 c'64, r32, r32, c'32., c'32. @}

    Interpret nonassignable `proportions` according to
    `decrease_durations_monotonically`::

        >>> print tuplettools.make_tuplet_from_duration_and_ratio(
        ...     Duration(3, 16), [5, -1, 5], avoid_dots=True,
        ...     decrease_durations_monotonically=False,
        ...     is_diminution=False)
        {@ 11:12 c'64, c'16, r64, c'64, c'16 @}

    Example set 2. Make augmented tuplet from `duration` and `proportions` and
    encourage dots::

        >>> tuplettools.make_tuplet_from_duration_and_ratio(
        ...     Duration(3, 16), [1, 1, 1, -1, -1], avoid_dots=False,
        ...     is_diminution=False)
        FixedDurationTuplet(3/16, [c'64., c'64., c'64., r64., r64.])

    Interpret nonassignable `proportions` according to
    `decrease_durations_monotonically`::

        >>> tuplettools.make_tuplet_from_duration_and_ratio(
        ...     Duration(3, 16), [5, -1, 5], avoid_dots=False,
        ...     decrease_durations_monotonically=False,
        ...     is_diminution=False)
        FixedDurationTuplet(3/16, [c'32..., r128., c'32...])

    Example set 3. Make diminished tuplet from `duration` and nonzero integer
    `proportions`.

    Return tupletted leaves strictly without dots when all `proportions` equal
    ``1``::

        >>> print tuplettools.make_tuplet_from_duration_and_ratio(
        ...     Duration(3, 16), [1, 1, 1, -1, -1], avoid_dots=True,
        ...     is_diminution=True)
        {@ 5:3 c'16, c'16, c'16, r16, r16 @}

    Allow tupletted leaves to return with dots when some `proportions` do not
    equal ``1``::

        >>> print tuplettools.make_tuplet_from_duration_and_ratio(
        ...     Duration(3, 16), [1, -2, -2, 3, 3], avoid_dots=True,
        ...     is_diminution=True)
        {@ 11:6 c'32, r16, r16, c'16., c'16. @}

    Interpret nonassignable `proportions` according to
    `decrease_durations_monotonically`::

        >>> print tuplettools.make_tuplet_from_duration_and_ratio(
        ...     Duration(3, 16), [5, -1, 5], avoid_dots=True,
        ...     decrease_durations_monotonically=False, 
        ...     is_diminution=True)
        {@ 11:6 c'32, c'8, r32, c'32, c'8 @}

    Example set 4. Make diminished tuplet from `duration` and `proportions` and
    encourage dots::

        >>> tuplettools.make_tuplet_from_duration_and_ratio(
        ...     Duration(3, 16), [1, 1, 1, -1, -1], avoid_dots=False,
        ...     is_diminution=True)
        FixedDurationTuplet(3/16, [c'32., c'32., c'32., r32., r32.])

    Interpret nonassignable `proportions` according to `direction`::

        >>> tuplettools.make_tuplet_from_duration_and_ratio(
        ...     Duration(3, 16), [5, -1, 5], avoid_dots=False,
        ...     decrease_durations_monotonically=False, 
        ...     is_diminution=True)
        FixedDurationTuplet(3/16, [c'16..., r64., c'16...])

    Reduce `proportions` relative to each other.

    Interpret negative `proportions` as rests.

    Return fixed-duration tuplet.
    '''
    from abjad.tools import tuplettools

    # coerce duration and proportions
    duration = durationtools.Duration(duration)
    proportions = mathtools.Ratio(proportions)

    # reduce proportions relative to each other
    proportions = sequencetools.divide_sequence_elements_by_greatest_common_divisor(proportions)

    # find basic prolated duration of note in tuplet
    basic_prolated_duration = duration / mathtools.weight(proportions)

    # find basic written duration of note in tuplet
    if avoid_dots:
        basic_written_duration = basic_prolated_duration.equal_or_greater_power_of_two
    else:
        basic_written_duration = basic_prolated_duration.equal_or_greater_assignable

    # find written duration of each note in tuplet
    written_durations = [x * basic_written_duration for x in proportions]

    # make tuplet leaves
    try:
        notes = [notetools.Note(0, x) if 0 < x else resttools.Rest(abs(x)) for x in written_durations]
    except AssignabilityError:
        denominator = duration._denominator
        note_durations = [durationtools.Duration(x, denominator) for x in proportions]
        pitches = [None if note_duration < 0 else 0 for note_duration in note_durations]
        leaf_durations = [abs(note_duration) for note_duration in note_durations]
        notes = leaftools.make_leaves(pitches, leaf_durations, 
            decrease_durations_monotonically=decrease_durations_monotonically)

    # make tuplet
    tuplet = tuplettools.FixedDurationTuplet(duration, notes)

    # fix tuplet contents if necessary
    tuplettools.fix_contents_of_tuplets_in_expr(tuplet)

    # switch prolation if necessary
    if not tuplet.multiplier == 1:
        #if prolation == 'diminution':
        if is_diminution:
            if not tuplet.is_diminution:
                tuplettools.change_augmented_tuplets_in_expr_to_diminished(tuplet)
        else:
            if tuplet.is_diminution:
                tuplettools.change_diminished_tuplets_in_expr_to_augmented(tuplet)

    # return tuplet
    return tuplet



from abjad.tools import chordtools
from abjad.tools import notetools
from abjad.tools.spannertools.Spanner import Spanner


class ComplexGlissandoSpanner(Spanner):
    r'''.. versionadded:: 2.9

    Abjad rest-skipping glissando spanner::

        >>> staff = Staff("c'16 r r g' r8 c'8")

    ::

        >>> spannertools.ComplexGlissandoSpanner(staff[:])
        ComplexGlissandoSpanner(c'16, r16, r16, g'16, r8, c'8)

    ::

        >>> f(staff)
        \new Staff {
            c'16 \glissando
            \once \override NoteColumn #'glissando-skip = ##t
            \once \override Rest #'transparent = ##t
            r16
            \once \override NoteColumn #'glissando-skip = ##t
            \once \override Rest #'transparent = ##t
            r16
            g'16 \glissando
            \once \override NoteColumn #'glissando-skip = ##t
            \once \override Rest #'transparent = ##t
            r8
            c'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Should be used with beamtools.BeamSpanner for best effect, along with
    an override of Stem #'stemlet-length, in order to generate stemlets over
    each invisible rest.

    Format nonlast leaves in spanner with LilyPond glissando command.

    Set all Rest instances to transparent.

    Set all NoteColumns filled with silences to be skipped by glissandi.

    Return `ComplexGlissandoSpanner` instance.
    '''

    ### INITIALIZER ###

    def __init__(self, components=None):
        Spanner.__init__(self, components)

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        pass

    def _format_before_leaf(self, leaf):
        result = []
        if not isinstance(leaf, (chordtools.Chord, notetools.Note)):
            result.append(r"\once \override NoteColumn #'glissando-skip = ##t")
            result.append(r"\once \override Rest #'transparent = ##t")
        return result

    def _format_right_of_leaf(self, leaf):
        '''Spanner contribution to right of leaf.'''
        result = []
        if not self._is_my_last_leaf(leaf) and isinstance(leaf, (chordtools.Chord, notetools.Note)):
            result.append(r'\glissando')
        return result



def get_marks_attached_to_components_in_expr(expr):
    r'''.. versionadded:: 2.9

    Get marks attached to components in `expr`::

        >>> staff = Staff(r"c'4 \pp d' \staccato e' \ff f' \staccato")

    ::

        \new Staff {
            c'4 \pp
            d'4 -\staccato
            e'4 \ff
            f'4 -\staccato
        }

    ::

        >>> for mark in marktools.get_marks_attached_to_components_in_expr(staff):
        ...     mark
        ...
        DynamicMark('pp')(c'4)
        Articulation('staccato')(d'4)
        DynamicMark('ff')(e'4) 
        Articulation('staccato')(f'4)

    Return tuple of zero or more marks.
    '''
    from abjad.tools import iterationtools
    from abjad.tools import marktools

    result = []
    for component in iterationtools.iterate_components_in_expr(expr):
        result.extend(marktools.get_marks_attached_to_component(component))

    return tuple(result)



from abjad.tools import componenttools


def iterate_namesakes_from_component(component, reverse=False, start=0, stop=None):
    r'''.. versionadded:: 1.1

    Iterate namesakes forward from `component`::

        >>> container = Container(Staff(notetools.make_repeated_notes(2)) * 2)
        >>> container.is_parallel = True
        >>> container[0].name = 'staff 1'
        >>> container[1].name = 'staff 2'
        >>> score = Score([])
        >>> score.is_parallel = False
        >>> score.extend(container * 2)
        >>> pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(score)

    ::

        >>> f(score)
        \new Score {
            <<
                \context Staff = "staff 1" {
                    c'8
                    d'8
                }
                \context Staff = "staff 2" {
                    e'8
                    f'8
                }
            >>
            <<
                \context Staff = "staff 1" {
                    g'8
                    a'8
                }
                \context Staff = "staff 2" {
                    b'8
                    c''8
                }
            >>
        }

    ::

        >>> for staff in iterationtools.iterate_namesakes_from_component(score[0][0]):
        ...     print staff.lilypond_format
        ...
        \context Staff = "staff 1" {
            c'8
            d'8
        }
        \context Staff = "staff 1" {
            g'8
            a'8
        }

    Iterate namesakes backward from `component`::

    ::

        >>> for staff in iterationtools.iterate_namesakes_from_component(
        ...     score[-1][0], reverse=True):
        ...     print staff.lilypond_format
        ...
        \context Staff = "staff 1" {
            g'8
            a'8
        }
        \context Staff = "staff 1" {
            c'8
            d'8
        }

    Return generator.
    '''
    from abjad.tools import iterationtools

    def _backward_helper(component):
        prev = componenttools.get_nth_component_in_time_order_from_component(component, -1)
        if prev is None:
            return
        dfs = iterationtools.iterate_components_depth_first(
            prev, capped=False, direction=Right)
        for node in dfs:
            if type(node) == type(component) and \
                node.parentage.parentage_signature == \
                component.parentage.parentage_signature:
                return node

    def _forward_helper(component):
        next_component = componenttools.get_nth_component_in_time_order_from_component(
            component, 1)
        if next_component is None:
            return
        dfs = iterationtools.iterate_components_depth_first(next_component, capped=False)
        for node in dfs:
            if type(node) == type(component) and \
                node.parentage.parentage_signature == component.parentage.parentage_signature:
                return node

    current_component = component
    total_components = 0

    if reverse:
        _helper = _backward_helper
    else:
        _helper = _forward_helper

    while current_component is not None:
        if start <= total_components:
            if stop is not None:
                if total_components < stop:
                    yield current_component
            else:
                yield current_component
        total_components += 1
        current_component = _helper(current_component)



import os


def clear_terminal():
    '''.. versionadded:: 2.0

    Run ``clear`` if OS is POSIX-compliant (UNIX / Linux / MacOS).

    Run ``cls`` if OS is not POSIX-compliant (Windows)::

        >>> iotools.clear_terminal() # doctest: +SKIP

    Return none.
    '''
    from abjad.tools import iotools

    if os.name == 'posix':
        iotools.spawn_subprocess('clear')
    else:
        iotools.spawn_subprocess('cls')



def get_effective_key_signature(component):
    r'''.. versionadded:: 2.0

    Get effective key signature of `component`::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> contexttools.KeySignatureMark('c', 'major')(staff)
        KeySignatureMark(NamedChromaticPitchClass('c'), Mode('major'))(Staff{4})

    ::

        >>> f(staff)
        \new Staff {
            \key c \major
            c'8
            d'8
            e'8
            f'8
        }

    ::

        >>> for note in staff:
        ...     note, contexttools.get_effective_key_signature(note)
        ...
        (Note("c'8"), KeySignatureMark(NamedChromaticPitchClass('c'), Mode('major'))(Staff{4}))
        (Note("d'8"), KeySignatureMark(NamedChromaticPitchClass('c'), Mode('major'))(Staff{4}))
        (Note("e'8"), KeySignatureMark(NamedChromaticPitchClass('c'), Mode('major'))(Staff{4}))
        (Note("f'8"), KeySignatureMark(NamedChromaticPitchClass('c'), Mode('major'))(Staff{4}))

    Return key signature mark or none.
    '''
    from abjad.tools import contexttools

    return contexttools.get_effective_context_mark(component, contexttools.KeySignatureMark)



import importlib
import os
from abjad.tools import abctools
from abjad.tools.documentationtools.APICrawler import APICrawler
from abjad.tools.documentationtools.ClassDocumenter import ClassDocumenter
from abjad.tools.documentationtools.FunctionDocumenter import FunctionDocumenter


class AbjadAPIGenerator(abctools.AbjadObject):
    '''Creates Abjad's API ReST:

        * writes ReST pages for individual classes and functions
        * writes the API index ReST
        * handles sorting tools packages into composition, manual-loading and unstable
        * handles ignoring private tools packages

    Returns `AbjadAPIGenerator` instance.
    '''

    ### CLASS ATTRIBUTES ###

    _api_title = 'Abjad API'

    _package_descriptions = {
        'core': 'Core composition packages',
        'demos': 'Demos and example packages',
        'internals': 'Abjad internal packages',
        'unstable': 'Unstable packages (load manually)',
    }

    _undocumented_packages = (
        'lilypondproxytools',
    )

    ### INITIALIZER ###

    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __call__(self, verbose=False):
        from abjad.tools import documentationtools

        if verbose:
            print 'Now making Sphinx TOCs ...'

        ignored_directories = ['.svn', 'test', '__pycache__']
        ignored_directories.extend(self._undocumented_packages)

        all_visited_modules = []
        for code_path, docs_path, package_prefix in self.path_definitions:
            if not os.path.exists(code_path):
                os.makedirs(code_path)
            if not os.path.exists(docs_path):
                os.makedirs(docs_path)
            crawler = APICrawler(code_path, docs_path, self.root_package,
                ignored_directories=ignored_directories, prefix=package_prefix)
            all_visited_modules.extend(crawler())
        package_dictionary, tools_package_dictionary = self._sort_modules(all_visited_modules)

        if verbose:
            print 'Now making API index ...'

        document = documentationtools.ReSTDocument()
        document.append(documentationtools.ReSTHeading(
            level=0,
            text=self._api_title,
            ))

        for package_group, packages in sorted(package_dictionary.items()):
            if packages:
                document.append(documentationtools.ReSTHeading(
                    level=1,
                    text=self._package_descriptions[package_group]
                    ))
                document.append(documentationtools.ReSTTOCDirective(
                    options={'maxdepth': 1},
                    ))
                for package_name, package in sorted(packages.items()):
                    document.extend(self._create_package_toc(package_name, package, tools_package_dictionary))

        f = open(self.docs_api_index_path, 'w')
        f.write(document.rest_format)
        f.close()

        if verbose:
            print ''
            print '... Done.'
            print ''

    ### PRIVATE METHODS ###

    def _create_package_toc(self, package_name, package_modules, tools_package_dictionary):
        from abjad.tools import documentationtools
        result = [documentationtools.ReSTHeading(
            level=2,
            text=':py:mod:`{} <{}>`'.format(package_name, tools_package_dictionary[package_name])
            )
        ]
        only_html = documentationtools.ReSTOnlyDirective(argument='html')
        only_latex = documentationtools.ReSTOnlyDirective(argument='latex')
        if package_modules['abstract_classes']:
            only_latex.append(documentationtools.ReSTHeading(
                level=3,
                text='Abstract Classes'
                ))
            toc_html = documentationtools.ReSTTOCDirective(
                options={'maxdepth': 1},
                )
            toc_latex = documentationtools.ReSTTOCDirective()
            for obj in package_modules['abstract_classes']:
                toc_entry = self._module_name_to_toc_entry(obj.module_name)
                toc_html.append(toc_entry)
                toc_latex.append(toc_entry)
            only_html.append(toc_html)
            only_latex.append(toc_latex)
        if package_modules['concrete_classes']:
            only_latex.append(documentationtools.ReSTHeading(
                level=3,
                text='Concrete Classes'
                ))
            if package_modules['abstract_classes']:
                only_html.append(documentationtools.ReSTHorizontalRule())
            toc_html = documentationtools.ReSTTOCDirective(
                options={'maxdepth': 1},
                )
            toc_latex = documentationtools.ReSTTOCDirective()
            for obj in package_modules['concrete_classes']:
                toc_entry = self._module_name_to_toc_entry(obj.module_name)
                toc_html.append(toc_entry)
                toc_latex.append(toc_entry)
            only_html.append(toc_html)
            only_latex.append(toc_latex)
        if package_modules['functions']:
            only_latex.append(documentationtools.ReSTHeading(
                level=3,
                text='Functions'
                ))
            toc_html = documentationtools.ReSTTOCDirective(
                options={'maxdepth': 1},
                )
            toc_latex = documentationtools.ReSTTOCDirective()
            if package_modules['concrete_classes'] or package_modules['abstract_classes']:
                only_html.append(documentationtools.ReSTHorizontalRule())
            for obj in package_modules['functions']:
                toc_entry = self._module_name_to_toc_entry(obj.module_name)
                toc_html.append(toc_entry)
                toc_latex.append(toc_entry)
            only_html.append(toc_html)
            only_latex.append(toc_latex)
        result.extend([only_html, only_latex])
        return result

    def _module_name_to_toc_entry(self, module_name):
        parts = module_name.split('.')[self.tools_package_path_index-1:-1]
        return '/'.join(parts)

    def _sort_modules(self, objects):
        packages = {}
        module_mapping = {}
        for obj in sorted(objects, key=lambda x: x.module_name):
            tools_package_name = obj.module_name.split('.')[self.tools_package_path_index]
            tools_package_path = '.'.join(obj.module_name.split('.')[:self.tools_package_path_index + 1])
            tools_package_module = importlib.import_module(tools_package_path)
            if tools_package_name not in module_mapping:
                module_mapping[tools_package_name] = tools_package_path
            if hasattr(tools_package_module, '_documentation_section'):
                declared_documentation_section = getattr(tools_package_module, '_documentation_section')
                if declared_documentation_section not in packages:
                    packages[declared_documentation_section] = {}
                collection = packages[declared_documentation_section]
            else:
                continue
            if tools_package_name not in collection:
                collection[tools_package_name] = {
                    'abstract_classes': [],
                    'concrete_classes': [],
                    'functions': []
                }
            if isinstance(obj, ClassDocumenter):
                if obj.is_abstract:
                    collection[tools_package_name]['abstract_classes'].append(obj)
                else:
                    collection[tools_package_name]['concrete_classes'].append(obj)
            else:
                collection[tools_package_name]['functions'].append(obj)
        return packages, module_mapping
        

    ### PUBLIC PROPERTIES ###

    @property
    def docs_api_index_path(self):
        '''Path to index.rst for Abjad API.'''
        from abjad import ABJCFG
        return os.path.join(ABJCFG.ABJAD_PATH, 'docs', 'source', 'api', 'index.rst')

    @property
    def package_prefix(self):
        return ('abjad.tools.', 'abjad.demos.')

    @property
    def path_definitions(self):
        '''Code path / docs path / package prefix triples.'''
        from abjad import ABJCFG
        return (
            (
                os.path.join(ABJCFG.ABJAD_PATH, 'tools'),
                os.path.join(ABJCFG.ABJAD_PATH, 'docs', 'source', 'api', 'tools'),
                'abjad.tools.',
            ),
            (
                os.path.join(ABJCFG.ABJAD_PATH, 'demos'),
                os.path.join(ABJCFG.ABJAD_PATH, 'docs', 'source', 'api', 'demos'),
                'abjad.demos.',
            ),
        )

    @property
    def root_package(self):
        return 'abjad'

    @property
    def tools_package_path_index(self):
        return 2



def iterate_semantic_voices_in_expr(expr, reverse=False, start=0, stop=None):
    r'''.. versionadded:: 2.0

    Iterate semantic voices forward in `expr`::

        >>> measures = measuretools.make_measures_with_full_measure_spacer_skips(
        ...     [(3, 8), (5, 16), (5, 16)])
        >>> time_signature_voice = Voice(measures)
        >>> time_signature_voice.name = 'TimeSignatuerVoice'
        >>> time_signature_voice.is_nonsemantic = True
        >>> music_voice = Voice("c'4. d'4 e'16 f'4 g'16")
        >>> music_voice.name = 'MusicVoice'
        >>> staff = Staff([time_signature_voice, music_voice])
        >>> staff.is_parallel = True

    ::

        >>> f(staff)
        \new Staff <<
            \context Voice = "TimeSignatuerVoice" {
                {
                    \time 3/8
                    s1 * 3/8
                }
                {
                    \time 5/16
                    s1 * 5/16
                }
                {
                    s1 * 5/16
                }
            }
            \context Voice = "MusicVoice" {
                c'4.
                d'4
                e'16
                f'4
                g'16
            }
        >>

        >>> for voice in iterationtools.iterate_semantic_voices_in_expr(staff):
        ...   voice
        Voice-"MusicVoice"{5}

    Iterate semantic voices backward in `expr`::

        >>> for voice in iterationtools.iterate_semantic_voices_in_expr(staff, reverse=True):
        ...   voice
        Voice-"MusicVoice"{5}

    Return generator.
    '''
    from abjad.tools import iterationtools

    for voice in iterationtools.iterate_voices_in_expr(
        expr, reverse=reverse, start=start, stop=stop):
        if not voice.is_nonsemantic:
            yield voice



def yield_all_partitions_of_integer(n):
    r'''.. versionadded:: 2.0

    Yield all partitions of positive integer `n` in descending lex order::

        >>> for partition in mathtools.yield_all_partitions_of_integer(7):
        ...     partition
        ...
        (7,)
        (6, 1)
        (5, 2)
        (5, 1, 1)
        (4, 3)
        (4, 2, 1)
        (4, 1, 1, 1)
        (3, 3, 1)
        (3, 2, 2)
        (3, 2, 1, 1)
        (3, 1, 1, 1, 1)
        (2, 2, 2, 1)
        (2, 2, 1, 1, 1)
        (2, 1, 1, 1, 1, 1)
        (1, 1, 1, 1, 1, 1, 1)

    Return generator of positive integer tuples of length at least ``1``.

    .. versionchanged:: 2.0
        renamed ``mathtools.integer_partitions()`` to
        ``mathtools.yield_all_partitions_of_integer()``.
    '''
    from abjad.tools import mathtools

    if not isinstance(n, int):
        raise TypeError('must be integer.')
    if not 0 < n:
        raise ValueError('must be positive.')

    partition = (n, )
    while partition is not None:
        yield partition
        partition = mathtools.next_integer_partition(partition)



from abjad.tools import componenttools
from abjad.tools import containertools
from abjad.tools import contexttools
from abjad.tools import durationtools
from abjad.tools import gracetools
from abjad.tools import mathtools
from abjad.tools import skiptools
from abjad.tools.spannertools.Spanner import Spanner


class MetricGridSpanner(Spanner):
    r'''Abjad metric grid spanner::

        >>> staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c'8")

    ::

        >>> spannertools.MetricGridSpanner(staff.leaves, time_signatures=[(1, 8), (1, 4)])
        MetricGridSpanner(c'8, d'8, e'8, f'8, g'8, a'8, b'8, c'8)

    ::

        >>> f(staff)
        \new Staff {
            \time 1/8
            c'8
            \time 1/4
            d'8
            e'8
            \time 1/8
            f'8
            \time 1/4
            g'8
            a'8
            \time 1/8
            b'8
            \time 1/4
            c'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Format leaves in spanner cyclically with `time_signatures`.

    Return metric grid spanner.
    '''

    def __init__(self, components=None, time_signatures=None):
        Spanner.__init__(self, components)
        self._time_signatures = time_signatures
        self.hide = False

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        new._time_signatures = self._time_signatures
        new.hide = self.hide

    def _format_after_leaf(self, leaf):
        result = []
        if hasattr(self, '_slicing_time_signaturesFound'):
            delattr(self, '_slicing_time_signaturesFound')
            result.append('>>')
        return result

    #FIXME: formatting is ridiculously slow.
    #         find a way to make it faster.
    # Tue Jan 13 12:05:43 EST 2009 [VA] using _slicing_time_signaturesFound boolean
    # flag now to improve performance time. Better but still not perfect.

    def _format_before_leaf(self, leaf):
        result = []
        if not self.hide:
            #time_signature = self._matching_time_signature(leaf)
            matching_time_signature = self._matching_time_signature(leaf)
            if matching_time_signature is None:
                time_signature = None
            else:
                time_signature, temp_hide = matching_time_signature
            #if time_signature and not getattr(time_signature, '_temp_hide', False):
            if time_signature and not temp_hide:
                result.append(time_signature.lilypond_format)
            #m = self._slicing_time_signatures(leaf)
            m = self._slicing_time_signatures(leaf)
            #m = [time_signature for time_signature in m if not getattr(time_signature, '_temp_hide', False)]
            m = [triple for triple in m if not triple[-1]]
            if m:
                # set self._slicing_time_signaturesFound as temporary flag so that
                # self._after does not have to recompute _slicing_time_signatures()
                self._slicing_time_signaturesFound = True
                result.append('<<')
                for time_signature, moffset, temp_hide in m:
                    s = skiptools.Skip(durationtools.Duration(1))
                    s.duration_multiplier = moffset - leaf.start_offset
                    numerator, denominator = time_signature.numerator, time_signature.denominator
                    mark = contexttools.TimeSignatureMark((numerator, denominator))(s)
                    mark._is_cosmetic_mark = True
                    container = containertools.Container([s])
                    result.append(container.lilypond_format)
        return result

    def _matching_time_signature(self, leaf):
        for m, moffset, temp_hide in self.time_signatures:
            if leaf.start_offset == moffset:
                return m, temp_hide

    def _slicing_time_signatures(self, leaf):
        '''Return the MetricStrip(s) that slices leaf, if any.
        '''
        for m, moffset, temp_hide in self.time_signatures:
            if leaf.start_offset < moffset:
                if moffset < leaf.stop_offset:
                    yield m, moffset, temp_hide
                else:
                    break

    ### PUBLIC PROPERTIES ###

    @apply
    def time_signatures():
        def fget(self):
            '''Get metric grid time_signatures::

                >>> staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c'8")
                >>> metric_grid_spanner = spannertools.MetricGridSpanner(
                ...     staff.leaves, time_signatures=[(1, 8), (1, 4)])

            Set metric grid time_signatures::

                >>> staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c'8")
                >>> metric_grid_spanner = spannertools.MetricGridSpanner(
                ...     staff.leaves, time_signatures=[(1, 8), (1, 4)])
                >>> metric_grid_spanner.time_signatures = [Duration(1, 4)]

            Set iterable.
            '''
            i = 0
            moffset = 0
            previous_time_signature = None
            #while moffset < self.duration:
            while moffset < self.prolated_duration:
                m = self._time_signatures[i % len(self._time_signatures)]
                m = contexttools.TimeSignatureMark(m)
                # new attribute
                if previous_time_signature and previous_time_signature == m:
                    #m.hide = True
                    #m._temp_hide = True
                    temp_hide = True
                else:
                    temp_hide = False
                #yield m
                yield m, moffset, temp_hide
                moffset += m.duration
                i += 1
                previous_time_signature = m
        def fset(self, time_signatures):
            assert isinstance(time_signatures, list)
            self._time_signatures = time_signatures
        return property(**locals())

    ### PUBLIC METHODS ###

    # TODO: looks like these tests are working now?
    def split_on_bar(self):
        '''Temporarily unavailable.
        '''

        leaves = [leaf for leaf in self.leaves if self.splitting_condition(leaf)]
        #self._debug(leaves, 'leaves')
        componenttools.split_components_at_offsets(leaves, [x[0].duration for x in self.time_signatures], 
            cyclic=True, fracture_spanners=False, tie_split_notes=True)
        #self._fuse_tied_leaves_within_measures()

    def splitting_condition(self, leaf):
        r'''User-definable boolean function to determine whether leaf should be split::

            >>> voice = Voice("c'4 r4 c'4")

        ::

            >>> f(voice)
            \new Voice {
                c'4
                r4
                c'4
            }

        ::

            >>> def cond(leaf):
            ...   if not isinstance(leaf, Rest): return True
            ...   else: return False
            >>> metric_grid_spanner = spannertools.MetricGridSpanner(
            ...     voice.leaves, [Duration(1, 8)])
            >>> metric_grid_spanner.splitting_condition = cond

        ::

            >>> metric_grid_spanner.split_on_bar()

        ::

            >>> f(voice)
            \new Voice {
                \time 1/8
                c'8 ~
                c'8
                r4
                c'8 ~
                c'8
            }

        Function defaults to return true.
        '''
        return True



from abjad.tools.datastructuretools.ObjectInventory import ObjectInventory
from abjad.tools.pitchtools.OctaveTranspositionMapping import OctaveTranspositionMapping


class OctaveTranspositionMappingInventory(ObjectInventory):
    '''.. versionadded:: 2.8

    Model of an ordered list of octave transposition mappings::

        >>> mapping_1 = pitchtools.OctaveTranspositionMapping([('[A0, C4)', 15), ('[C4, C8)', 27)])
        >>> mapping_2 = pitchtools.OctaveTranspositionMapping([('[A0, C8]', -18)])
        >>> inventory = pitchtools.OctaveTranspositionMappingInventory([mapping_1, mapping_2])

    ::

        >>> z(inventory)
        pitchtools.OctaveTranspositionMappingInventory([
            pitchtools.OctaveTranspositionMapping([
                pitchtools.OctaveTranspositionMappingComponent(
                    pitchtools.PitchRange(
                        '[A0, C4)'
                        ),
                    pitchtools.NumberedChromaticPitch(
                        15
                        )
                    ),
                pitchtools.OctaveTranspositionMappingComponent(
                    pitchtools.PitchRange(
                        '[C4, C8)'
                        ),
                    pitchtools.NumberedChromaticPitch(
                        27
                        )
                    )
                ]),
            pitchtools.OctaveTranspositionMapping([
                pitchtools.OctaveTranspositionMappingComponent(
                    pitchtools.PitchRange(
                        '[A0, C8]'
                        ),
                    pitchtools.NumberedChromaticPitch(
                        -18
                        )
                    )
                ])
            ])

    Octave transposition mapping inventories implement list interface and are mutable.
    '''

    ### READ-ONLY PRIVATE PROPERTIES ###
    
    @property
    def _item_callable(self):
        return OctaveTranspositionMapping



from abjad import *


def test_rhythmtreetools_parse_rtm_syntax_01():

    rtm = '(3 (1 (3 (1 (3 (1 (3 (1 1 1 1))))))))'
    result = rhythmtreetools.parse_rtm_syntax(rtm)

    r'''
    \fraction \times 3/4 {
        c'4
        \fraction \times 3/4 {
            c'4
            \fraction \times 3/4 {
                c'4
                \fraction \times 3/4 {
                    c'4
                    c'4
                    c'4
                    c'4
                }
            }
        }
    }
    '''

    assert result.lilypond_format == "\\fraction \\times 3/4 {\n\tc'4\n\t\\fraction \\times 3/4 {\n\t\tc'4\n\t\t\\fraction \\times 3/4 {\n\t\t\tc'4\n\t\t\t\\fraction \\times 3/4 {\n\t\t\t\tc'4\n\t\t\t\tc'4\n\t\t\t\tc'4\n\t\t\t\tc'4\n\t\t\t}\n\t\t}\n\t}\n}"




def all_are_chromatic_pitch_class_name_octave_number_pairs(expr):
    '''.. versionadded:: 1.1

    True when all elements of `expr` are pitch tokens. Otherwise false::

        >>> pitchtools.all_are_chromatic_pitch_class_name_octave_number_pairs(
        ... [('c', 4), ('d', 4), pitchtools.NamedChromaticPitch('e', 4)])
        True

    Return boolean.
    '''
    from abjad.tools import pitchtools

    if isinstance(expr, (list, tuple, set)):
        if all([pitchtools.is_named_chromatic_pitch_token(x) for x in expr]):
            return True
    return False



from abjad.tools import componenttools
from abjad.tools import durationtools


# TODO: implement corresponding function to rest left half
def rest_leaf_at_offset(leaf, offset):
    r'''.. versionadded:: 1.1

    Split `leaf` at `offset` and rest right half::

        >>> staff = Staff("c'8 ( d'8 e'8 f'8 )")

    ::

        >>> f(staff)
        \new Staff {
            c'8 (
            d'8
            e'8
            f'8 )
        }

    ::

        >>> leaftools.rest_leaf_at_offset(staff.leaves[1], (1, 32))
        ([Note("d'32")], [Note("d'16.")])

    ::

        >>> f(staff)
        \new Staff {
            c'8 (
            d'32
            r16.
            e'8
            f'8 )
        }

    Return list of leaves to left of `offset`
    together with list of leaves to right of `offset`.

    .. versionchanged:: 2.0
        renamed ``leaftools.shorten()`` to
        ``leaftools.rest_leaf_at_offset()``.
    '''
    from abjad.tools import resttools

    offset = durationtools.Offset(offset)

    left, right = componenttools.split_component_at_offset(
        leaf, offset, fracture_spanners=False, tie_split_notes=False)

    for leaf in right:
        rest = resttools.Rest(leaf)
        componenttools.move_parentage_and_spanners_from_components_to_components([leaf], [rest])

    return left, right



from abjad import *


def test_labeltools_label_notes_in_expr_with_note_indices_01():

    staff = Staff("c'8 d'8 r8 r8 g'8 a'8 r8 c''8")
    labeltools.label_notes_in_expr_with_note_indices(staff)

    r'''
    \new Staff {
        c'8
            _ \markup {
                \small
                    0
                }
        d'8
            _ \markup {
                \small
                    1
                }
        r8
        r8
        g'8
            _ \markup {
                \small
                    2
                }
        a'8
            _ \markup {
                \small
                    3
                }
        r8
        c''8
            _ \markup {
                \small
                    4
                }
    }
    '''

    assert staff.lilypond_format == "\\new Staff {\n\tc'8\n\t\t_ \\markup {\n\t\t\t\\small\n\t\t\t\t0\n\t\t\t}\n\td'8\n\t\t_ \\markup {\n\t\t\t\\small\n\t\t\t\t1\n\t\t\t}\n\tr8\n\tr8\n\tg'8\n\t\t_ \\markup {\n\t\t\t\\small\n\t\t\t\t2\n\t\t\t}\n\ta'8\n\t\t_ \\markup {\n\t\t\t\\small\n\t\t\t\t3\n\t\t\t}\n\tr8\n\tc''8\n\t\t_ \\markup {\n\t\t\t\\small\n\t\t\t\t4\n\t\t\t}\n}"




from abjad.tools import leaftools


def make_tied_note(pitch, duration, decrease_durations_monotonically=True):
    '''Returns a list of notes to fill the given duration.

    Notes returned are tie-spanned.
    '''
    from abjad.tools import notetools

    return leaftools.make_tied_leaf(
        notetools.Note, duration, 
        decrease_durations_monotonically=decrease_durations_monotonically, 
        pitches=pitch, tie_parts=True)



from abjad.tools import durationtools
from abjad.tools import sequencetools
from abjad.tools.abctools import ImmutableAbjadObject


class QEventSequence(tuple, ImmutableAbjadObject):
    '''A well-formed sequence of ``QEvent`` instances, containing only
    ``PitchedQEvents`` and ``SilentQEvents``, and terminating with a
    single ``TerminalQEvent`` instance.

    ``QEventSequence`` is the primary input to the ``Quantizer``.

    ``QEventSequence`` provides a number of convenience functions to
    assist with instantiating new sequences:

    ::

        >>> durations = (1000, -500, 1250, -500, 750)

    ::

        >>> sequence = quantizationtools.QEventSequence.from_millisecond_durations(
        ...     durations)

    ::

        >>> for q_event in sequence:
        ...     q_event
        quantizationtools.PitchedQEvent(
            durationtools.Offset(0, 1),
            (NamedChromaticPitch("c'"),),
            attachments=()
            )
        quantizationtools.SilentQEvent(
            durationtools.Offset(1000, 1),
            attachments=()
            )
        quantizationtools.PitchedQEvent(
            durationtools.Offset(1500, 1),
            (NamedChromaticPitch("c'"),),
            attachments=()
            )
        quantizationtools.SilentQEvent(
            durationtools.Offset(2750, 1),
            attachments=()
            )
        quantizationtools.PitchedQEvent(
            durationtools.Offset(3250, 1),
            (NamedChromaticPitch("c'"),),
            attachments=()
            )
        quantizationtools.TerminalQEvent(
            durationtools.Offset(4000, 1)
            )

    Return ``QEventSequence`` instance.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __new__(klass, args):
        from abjad.tools import quantizationtools
        klasses = (quantizationtools.PitchedQEvent, quantizationtools.SilentQEvent)
        assert 1 < len(args)
        assert all([isinstance(x, klasses) for x in args[:-1]])
        assert isinstance(args[-1], quantizationtools.TerminalQEvent)
        assert sequencetools.is_monotonically_increasing_sequence([x.offset for x in args])
        assert 0 <= args[0].offset
        return tuple.__new__(klass, args)

    ### SPECIAL METHODS ###

    def __repr__(self):
        return '{}({})'.format(self._class_name, tuple.__repr__(self))

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def duration_in_ms(self):
        '''The total duration in milliseconds of the ``QEventSequence``:

        ::

            >>> sequence.duration_in_ms
            Duration(4000, 1)

        Return ``Duration`` instance.
        '''
        return durationtools.Duration(self[-1].offset)

    ### PUBLIC METHODS ###

    @classmethod
    def from_millisecond_durations(klass, durations, fuse_silences=False):
        '''Convert a sequence of millisecond durations ``durations`` into
        a ``QEventSequence``:

        ::

            >>> durations = [-250, 500, -1000, 1250, -1000]

        ::

            >>> sequence = quantizationtools.QEventSequence.from_millisecond_durations(
            ...     durations)

        ::

            >>> for q_event in sequence:
            ...     q_event
            ...
            quantizationtools.SilentQEvent(
                durationtools.Offset(0, 1),
                attachments=()
                )
            quantizationtools.PitchedQEvent(
                durationtools.Offset(250, 1),
                (NamedChromaticPitch("c'"),),
                attachments=()
                )
            quantizationtools.SilentQEvent(
                durationtools.Offset(750, 1),
                attachments=()
                )
            quantizationtools.PitchedQEvent(
                durationtools.Offset(1750, 1),
                (NamedChromaticPitch("c'"),),
                attachments=()
                )
            quantizationtools.SilentQEvent(
                durationtools.Offset(3000, 1),
                attachments=()
                )
            quantizationtools.TerminalQEvent(
                durationtools.Offset(4000, 1)
                )

        Return ``QEventSequence`` instance.
        '''
        from abjad.tools.quantizationtools import milliseconds_to_q_events
        return klass(milliseconds_to_q_events(durations, fuse_silences))

    @classmethod
    def from_millisecond_offsets(klass, offsets):
        '''Convert millisecond offsets ``offsets`` into a ``QEventSequence``:

        ::
        
            >>> offsets = [0, 250, 750, 1750, 3000, 4000]

        ::

            >>> sequence = quantizationtools.QEventSequence.from_millisecond_offsets(
            ...     offsets)

        ::

            >>> for q_event in sequence:
            ...     q_event
            ...
            quantizationtools.PitchedQEvent(
                durationtools.Offset(0, 1),
                (NamedChromaticPitch("c'"),),
                attachments=()
                )
            quantizationtools.PitchedQEvent(
                durationtools.Offset(250, 1),
                (NamedChromaticPitch("c'"),),
                attachments=()
                )
            quantizationtools.PitchedQEvent(
                durationtools.Offset(750, 1),
                (NamedChromaticPitch("c'"),),
                attachments=()
                )
            quantizationtools.PitchedQEvent(
                durationtools.Offset(1750, 1),
                (NamedChromaticPitch("c'"),),
                attachments=()
                )
            quantizationtools.PitchedQEvent(
                durationtools.Offset(3000, 1),
                (NamedChromaticPitch("c'"),),
                attachments=()
                )
            quantizationtools.TerminalQEvent(
                durationtools.Offset(4000, 1)
                )

        Return ``QEventSequence`` instance.
        '''
        from abjad.tools import quantizationtools
        q_events = [quantizationtools.PitchedQEvent(x, [0]) for x in offsets[:-1]]
        q_events.append(quantizationtools.TerminalQEvent(offsets[-1]))
        return klass(q_events)

    @classmethod
    def from_millisecond_pitch_pairs(klass, pairs):
        '''Convert millisecond-duration:pitch pairs ``pairs`` into a ``QEventSequence``:

        ::

            >>> durations = [250, 500, 1000, 1250, 1000]
            >>> pitches = [(0,), None, (2, 3), None, (1,)]
            >>> pairs = zip(durations, pitches)

        ::

            >>> sequence = quantizationtools.QEventSequence.from_millisecond_pitch_pairs(
            ...     pairs)

        ::

            >>> for q_event in sequence:
            ...     q_event
            ...
            quantizationtools.PitchedQEvent(
                durationtools.Offset(0, 1),
                (NamedChromaticPitch("c'"),),
                attachments=()
                )
            quantizationtools.SilentQEvent(
                durationtools.Offset(250, 1),
                attachments=()
                )
            quantizationtools.PitchedQEvent(
                durationtools.Offset(750, 1),
                (NamedChromaticPitch("d'"), NamedChromaticPitch("ef'")),
                attachments=()
                )
            quantizationtools.SilentQEvent(
                durationtools.Offset(1750, 1),
                attachments=()
                )
            quantizationtools.PitchedQEvent(
                durationtools.Offset(3000, 1),
                (NamedChromaticPitch("cs'"),),
                attachments=()
                )
            quantizationtools.TerminalQEvent(
                durationtools.Offset(4000, 1)
                )

        Return ``QEventSequence`` instance.
        '''
        from abjad.tools.quantizationtools import millisecond_pitch_pairs_to_q_events
        return klass(millisecond_pitch_pairs_to_q_events(pairs))

    @classmethod
    def from_tempo_scaled_durations(klass, durations, tempo=None):
        '''Convert ``durations``, scaled by ``tempo`` into a ``QEventSequence``:

        ::

            >>> tempo = contexttools.TempoMark((1, 4), 174)
            >>> durations = [(1, 4), (-3, 16), (1, 16), (-1, 2)]

        ::

            >>> sequence = quantizationtools.QEventSequence.from_tempo_scaled_durations(
            ...     durations, tempo=tempo)

        ::

            >>> for q_event in sequence:
            ...     q_event
            ...
            quantizationtools.PitchedQEvent(
                durationtools.Offset(0, 1),
                (NamedChromaticPitch("c'"),),
                attachments=()
                )
            quantizationtools.SilentQEvent(
                durationtools.Offset(10000, 29),
                attachments=()
                )
            quantizationtools.PitchedQEvent(
                durationtools.Offset(17500, 29),
                (NamedChromaticPitch("c'"),),
                attachments=()
                )
            quantizationtools.SilentQEvent(
                durationtools.Offset(20000, 29),
                attachments=()
                )
            quantizationtools.TerminalQEvent(
                durationtools.Offset(40000, 29)
                )

        Return ``QEventSequence`` instance.
        '''
        from abjad.tools.quantizationtools import tempo_scaled_durations_to_q_events
        return klass(tempo_scaled_durations_to_q_events(durations, tempo))

    @classmethod
    def from_tempo_scaled_leaves(klass, leaves, tempo=None):
        '''Convert ``leaves``, optionally with ``tempo`` into a ``QEventSequence``:

        ::

            >>> staff = Staff("c'4 <d' fs'>8. r16 gqs'2")
            >>> tempo = contexttools.TempoMark((1, 4), 72)

        ::

            >>> sequence = quantizationtools.QEventSequence.from_tempo_scaled_leaves(
            ...     staff.leaves, tempo=tempo)

        ::

            >>> for q_event in sequence:
            ...     q_event
            ...
            quantizationtools.PitchedQEvent(
                durationtools.Offset(0, 1),
                (NamedChromaticPitch("c'"),),
                attachments=()
                )
            quantizationtools.PitchedQEvent(
                durationtools.Offset(2500, 3),
                (NamedChromaticPitch("d'"), NamedChromaticPitch("fs'")),
                attachments=()
                )
            quantizationtools.SilentQEvent(
                durationtools.Offset(4375, 3),
                attachments=()
                )
            quantizationtools.PitchedQEvent(
                durationtools.Offset(5000, 3),
                (NamedChromaticPitch("gqs'"),),
                attachments=()
                )
            quantizationtools.TerminalQEvent(
                durationtools.Offset(10000, 3)
                )

        If ``tempo`` is ``None``, all leaves in ``leaves`` must have an effective,
        non-imprecise tempo.  The millisecond-duration of each leaf will be
        determined by its effective tempo.

        Return ``QEventSequence`` instance.
        '''
        from abjad.tools.quantizationtools import tempo_scaled_leaves_to_q_events
        return klass(tempo_scaled_leaves_to_q_events(leaves, tempo))



from abjad.tools import contexttools
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools.instrumenttools._Trombone._Trombone import _Trombone


class BassTrombone(_Trombone):
    r'''.. versionadded:: 2.0

    Abjad model of the tenor trombone::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> contexttools.ClefMark('bass')(staff)
        ClefMark('bass')(Staff{4})

    ::

        >>> instrumenttools.BassTrombone()(staff)
        BassTrombone()(Staff{4})

    ::

        >>> f(staff)
        \new Staff {
            \clef "bass"
            \set Staff.instrumentName = \markup { Bass trombone }
            \set Staff.shortInstrumentName = \markup { Bass trb. }
            c'8
            d'8
            e'8
            f'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    The tenor trombone targets staff context by default.
    '''

    def __init__(self, **kwargs):
        _Trombone.__init__(self, **kwargs)
        self._default_instrument_name = 'bass trombone'
        self._default_short_instrument_name = 'bass trb.'
        self._is_primary_instrument = False
        self.sounding_pitch_of_written_middle_c = pitchtools.NamedChromaticPitch("c'")
        self.primary_clefs = [contexttools.ClefMark('bass')]
        self._copy_primary_clefs_to_all_clefs()
        self._traditional_pitch_range = pitchtools.PitchRange('[C2, F4]')



from abjad.tools import mathtools


def durations_to_nonreduced_fractions_with_common_denominator(durations):
    '''.. versionadded:: 2.0

    Change `durations` to nonreduced fractions with least common denominator::

        >>> durations = [Duration(2, 4), 3, '8.', (5, 16)]
        >>> for x in durationtools.durations_to_nonreduced_fractions_with_common_denominator(
        ...     durations):
        ...     x
        ...
        NonreducedFraction(8, 16)
        NonreducedFraction(48, 16) 
        NonreducedFraction(3, 16) 
        NonreducedFraction(5, 16)

    Return new object of `durations` type.
    '''
    from abjad.tools import durationtools

    durations = [durationtools.Duration(x) for x in durations]
    denominators = [duration.denominator for duration in durations]
    lcd = mathtools.least_common_multiple(*denominators)

    nonreduced_fractions = [mathtools.NonreducedFraction(x).with_denominator(lcd) for x in durations]
    result = type(durations)(nonreduced_fractions)

    return result



from abjad.tools import componenttools


def repeat_leaf(leaf, total=1):
    r'''.. versionadded:: 1.1

    Repeat `leaf` and extend spanners::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> beamtools.BeamSpanner(staff.leaves)
        BeamSpanner(c'8, d'8, e'8, f'8)
        >>> f(staff)
        \new Staff {
            c'8 [
            d'8
            e'8
            f'8 ]
        }

    ::

        >>> leaftools.repeat_leaf(staff[0], total=3)

    ::

        >>> f(staff)
        \new Staff {
            c'8 [
            c'8
            c'8
            d'8
            e'8
            f'8 ]
        }

    Preserve `leaf` written duration.

    Preserve parentage and spanners.

    Return none.

    .. versionchanged:: 2.0
        renamed ``leaftools.clone_and_splice_leaf()`` to
        ``leaftools.repeat_leaf()``.
    '''

    componenttools.extend_in_parent_of_component(
        leaf, 
        componenttools.copy_components_and_remove_spanners([leaf], total - 1), 
        grow_spanners=True)



def explode_intervals_uncompactly(intervals):
    '''Explode the intervals in `intervals` into n non-overlapping trees,
    where n is the maximum depth of `intervals`.

    The algorithm will attempt to insert the exploded intervals
    cyclically, making its insertion attempt at the next resultant tree
    in the array, rather than always beginning its search from index 0.

    Return list of `TimeIntervalTree` instances.
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    depth_tree = timeintervaltools.compute_depth_of_intervals(tree)
    max_depth = max([x['depth'] for x in depth_tree])
    layers = [[] for i in range(max_depth)]

    offset = 0
    for interval in tree:
        for i in range(max_depth):
            layer = layers[(i + offset) % max_depth]
            if not len(layer):
                layer.append(interval)
                offset = i + 1
                break
            elif not layer[-1].is_overlapped_by_interval(interval):
                layer.append(interval)
                offset = i + 1
                break

    return [timeintervaltools.TimeIntervalTree(layer) for layer in layers]



from abjad.tools import componenttools


def get_spanners_covered_by_components(components):
    '''.. versionadded:: 1.1

    Get spanners covered by `components`.

    Return unordered set of  spanners completely contained
    within the time bounds of thread-contiguous components.

    A spanner `p` is covered by timespan `t` when and only when
    ``t.start_ofset <= p.start_offset and p.stop_offset <= t.stop_offset``.

    .. versionchanged:: 2.0
        renamed ``spannertools.get_covered()`` to
        ``spannertools.get_spanners_covered_by_components()``.
    '''
    from abjad.tools import spannertools

    assert componenttools.all_are_thread_contiguous_components(components)

    if not len(components):
        return set([])

    first, last = components[0], components[-1]
    components_begin = first.start_offset
    components_end = last.stop_offset

    result = spannertools.get_spanners_contained_by_components(components)
    for spanner in list(result):
        if spanner.start_offset < components_begin or components_end < spanner.stop_offset:
            result.discard(spanner)

    return result



def detach_grace_containers_attached_to_leaves_in_expr(expr, kind=None):
    r'''.. versionadded:: 2.9

    Detach grace containers attached to leaves in `expr`::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> grace_container = gracetools.GraceContainer([Note("cs'16")], kind='grace')
        >>> grace_container(staff[1])
        Note("d'8")

    ::

        >>> f(staff)
        \new Staff {
            c'8
            \grace {
                cs'16
            }
            d'8
            e'8
            f'8
        }

    ::

        >>> gracetools.detach_grace_containers_attached_to_leaves_in_expr(staff)
        (GraceContainer(),)

    ::

        >>> f(staff)
        \new Staff {
            c'8
            d'8
            e'8
            f'8
        }

    Return tuple of zero or more grace containers.
    '''
    from abjad.tools import gracetools
    from abjad.tools import iterationtools

    result = []
    for leaf in iterationtools.iterate_leaves_in_expr(expr):
        result.extend(gracetools.detach_grace_containers_attached_to_leaf(leaf, kind=kind))

    return tuple(result)



def get_nth_sibling_from_component(component, n):
    r'''.. versionadded:: 2.9

    Get nth sibling from `component`::

        >>> staff = Staff("c' d' e' f'")
    
    ::

        >>> f(staff)
        \new Staff {
            c'4
            d'4
            e'4
            f'4
        }

    ::

        >>> staff[1]
        Note("d'4")

    Return sibling to the right of `component` for positive `n`::

        >>> componenttools.get_nth_sibling_from_component(staff[1], 1)
        Note("e'4")

    Return sibling to the left of `component` for negative `n`::

        >>> componenttools.get_nth_sibling_from_component(staff[1], -1)
        Note("c'4")

    Return `component` when `n` is ``0``::

        >>> componenttools.get_nth_sibling_from_component(staff[1], 0)
        Note("d'4")

    Return none when `n` is out of range::

        >>> componenttools.get_nth_sibling_from_component(staff[1], 99) is None
        True

    Return none when `component` has no parent::

        >>> componenttools.get_nth_sibling_from_component(staff, 1) is None
        True

    Return component or none.
    '''

    if n == 0:
        return component
    elif 0 < n:
        if component.parent is not None:
            if not component.parent.is_parallel:    
                index = component.parent.index(component)
                if index + n < len(component.parent):
                    return component.parent[index + n]
    elif n < 0:
        if component.parent is not None:
            if not component.parent.is_parallel:
                index = component.parent.index(component)
                if 0 <= index + n:
                    return component.parent[index + n]



def sum_duration_of_components(components, preprolated=False, in_seconds=False):
    r'''.. versionadded:: 1.1

    Sum duration of `components`::

        >>> tuplet = Tuplet((2, 3), "c'8 d'8 e'8")
        >>> staff = Staff([tuplet])
        >>> score = Score([staff])
        >>> contexttools.TempoMark(Duration(1, 4), 48)(tuplet[0])
        TempoMark(Duration(1, 4), 48)(c'8)

    ::

        >>> f(score)
        \new Score <<
            \new Staff {
                \times 2/3 {
                    \tempo 4=48
                    c'8
                    d'8
                    e'8
                }
            }
        >>

    ::

        >>> show(score) # doctest: +SKIP

    Example 1. Sum duration of components::

        >>> componenttools.sum_duration_of_components(tuplet[:])
        Duration(1, 4)

    Example 2. Sum preprolated duration of components::

        >>> componenttools.sum_duration_of_components(tuplet[:], preprolated=True)
        Duration(3, 8)

    Example 3. Sum duration of components in seconds::

        >>> componenttools.sum_duration_of_components(tuplet[:], in_seconds=True)
        Duration(5, 4)

    Return duration.
    '''

    if in_seconds:
        return sum([component.duration_in_seconds for component in components])
    elif preprolated:
        return sum([component.preprolated_duration for component in components])
    else:
        return sum([component.prolated_duration for component in components])



import types
from abjad.tools import selectiontools


def all_are_contiguous_components_in_same_thread(expr, klasses=None, allow_orphans=True):
    '''.. versionadded:: 1.1

    True when elements in `expr` are all contiguous components in same thread.
    Otherwise false::

        >>> staff = Staff("c'8 d'8 e'8")
        >>> componenttools.all_are_contiguous_components_in_same_thread(staff.leaves)
        True

    True when elements in `expr` are all contiguous `klasses` in same thread.
    Otherwise false::

        >>> staff = Staff("c'8 d'8 e'8")
        >>> componenttools.all_are_contiguous_components_in_same_thread(staff.leaves, klasses=Note)
        True

    Return boolean.
    '''
    from abjad.tools import componenttools

    if not isinstance(expr, (list, tuple, types.GeneratorType, selectiontools.Selection)):
        return False

    if klasses is None:
        klasses = componenttools.Component

    if len(expr) == 0:
        return True

    first = expr[0]
    if not isinstance(first, klasses):
        return False

    orphan_components = True
    if not first.parentage.is_orphan:
        orphan_components = False

    same_thread = True
    strictly_contiguous = True

    first_signature = first.parentage.containment_signature
    prev = first
    for cur in expr[1:]:
        if not isinstance(cur, klasses):
            return False
        if not cur.parentage.is_orphan:
            orphan_components = False
        current_signature = cur.parentage.containment_signature
        if not current_signature == first_signature:
            same_thread = False
        if not componenttools.is_immediate_temporal_successor_of_component(prev, cur):
            strictly_contiguous = False
        if (not allow_orphans or (allow_orphans and not orphan_components)) and \
            (not same_thread or not strictly_contiguous):
            return False
        prev = cur

    return True



def apply_beam_spanners_to_measures_in_expr(expr):
    r'''.. versionadded:: 1.1

    Apply beam spanners to measures in `expr`::

        >>> staff = Staff(r"abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |")

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> beamtools.apply_beam_spanners_to_measures_in_expr(staff)
        [BeamSpanner(|2/8(2)|), BeamSpanner(|2/8(2)|)]

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'8 [
                d'8 ]
            }
            {
                e'8 [
                f'8 ]
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Return list of beams created.

    .. versionchanged:: 2.0
        renamed ``measuretools.beam()`` to
        ``beamtools.apply_beam_spanners_to_measures_in_expr()``.

    .. versionchanged:: 2.9
        renamed ``measuretools.apply_beam_spanners_to_measures_in_expr()`` to
        ``beamtools.apply_beam_spanners_to_measures_in_expr()``.
    '''
    from abjad.tools import beamtools
    from abjad.tools import iterationtools

    # init beams created
    beams_created = []

    # apply beam spanners to measures in expr
    for measure in iterationtools.iterate_measures_in_expr(expr):
        beam = beamtools.BeamSpanner(measure)
        beams_created.append(beam)

    # return beams created
    return beams_created



from abjad import *


def test_iterationtools_iterate_leaves_in_expr_01():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")

    r'''
    \new Staff {
      {
            \time 2/8
            c'8
            d'8
      }
      {
            \time 2/8
            e'8
            f'8
      }
      {
            \time 2/8
            g'8
            a'8
      }
    }
    '''

    generator = iterationtools.iterate_leaves_in_expr(staff, reverse=True)
    leaves = list(generator)

    assert leaves[0] is staff[2][1]
    assert leaves[1] is staff[2][0]
    assert leaves[2] is staff[1][1]
    assert leaves[3] is staff[1][0]
    assert leaves[4] is staff[0][1]
    assert leaves[5] is staff[0][0]


def test_iterationtools_iterate_leaves_in_expr_02():
    '''Optional start and stop keyword parameters.'''

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")

    leaves = list(iterationtools.iterate_leaves_in_expr(staff, start=3, reverse=True))
    assert leaves[0] is staff[1][0]
    assert leaves[1] is staff[0][1]
    assert leaves[2] is staff[0][0]
    assert len(leaves) == 3

    leaves = list(iterationtools.iterate_leaves_in_expr(staff, start=0, stop=3, reverse=True))
    assert leaves[0] is staff[2][1]
    assert leaves[1] is staff[2][0]
    assert leaves[2] is staff[1][1]
    assert len(leaves) == 3

    leaves = list(iterationtools.iterate_leaves_in_expr(staff, start=2, stop=4, reverse=True))
    assert leaves[0] is staff[1][1]
    assert leaves[1] is staff[1][0]
    assert len(leaves) == 2


def test_iterationtools_iterate_leaves_in_expr_03():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")

    r'''
    \new Staff {
      {
            \time 2/8
            c'8
            d'8
      }
      {
            \time 2/8
            e'8
            f'8
      }
      {
            \time 2/8
            g'8
            a'8
      }
    }
    '''

    generator = iterationtools.iterate_leaves_in_expr(staff)
    leaves = list(generator)

    assert leaves[0] is staff[0][0]
    assert leaves[1] is staff[0][1]
    assert leaves[2] is staff[1][0]
    assert leaves[3] is staff[1][1]
    assert leaves[4] is staff[2][0]
    assert leaves[5] is staff[2][1]


def test_iterationtools_iterate_leaves_in_expr_04():
    '''Optional start and stop keyword parameters.'''

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")

    leaves = list(iterationtools.iterate_leaves_in_expr(staff, start=3))
    assert leaves[0] is staff[1][1]
    assert leaves[1] is staff[2][0]
    assert leaves[2] is staff[2][1]
    assert len(leaves) == 3

    leaves = list(iterationtools.iterate_leaves_in_expr(staff, start=0, stop=3))
    assert leaves[0] is staff[0][0]
    assert leaves[1] is staff[0][1]
    assert leaves[2] is staff[1][0]
    assert len(leaves) == 3

    leaves = list(iterationtools.iterate_leaves_in_expr(staff, start=2, stop=4))
    assert leaves[0] is staff[1][0]
    assert leaves[1] is staff[1][1]
    assert len(leaves) == 2



from abjad import *
import py


def test_LilyPondParser__lilypondfile__HeaderBlock_01():

    string = r'''
    globalvariable = "This is a global variable."
    \header {
        globalvariable = "This overrides the global variable"
        localvariable = "and this is a local variable."
        title = \markup { \globalvariable \localvariable }
        something = #4
    }
    \score {
        \new Staff { c'4 ^ \markup { \globalvariable } }
    }
    '''

    result = p(string)

    assert isinstance(result, lilypondfiletools.LilyPondFile)
    assert len(result) == 2

    assert result[0].lilypond_format == '\\header {\n\tglobalvariable = #"This overrides the global variable"\n\tlocalvariable = #"and this is a local variable."\n\tsomething = #4\n\ttitle = \\markup {\n\t\t"This overrides the global variable"\n\t\t"and this is a local variable."\n\t\t}\n}'

    r'''
    \header {
        globalvariable = #"This overrides the global variable"
        localvariable = #"and this is a local variable."
        something = #4
        title = \markup {
            "This overrides the global variable"
            "and this is a local variable."
        }
    }
    '''

    assert result[1].lilypond_format == '\\score {\n\t\\new Staff {\n\t\tc\'4 ^ \\markup { This is a global variable. }\n\t}\n}'

    r'''
    \score {
        \new Staff {
            c'4 ^ \markup { This is a global variable. }
        }
    }
    '''



import types


def list_abjad_environment_variables():
    '''.. versionadded:: 1.1

    List Abjad environment variables.

    Return tuple of zero or more environment variable / setting pairs.

    Abjad environment variables are defined in 
    ``abjad/tools/configurationtools/AbjadConfig/AbjadConfig.py``.

    .. versionchanged:: 2.0
        renamed ``configurationtools.list_settings()`` to
        ``configurationtools.list_abjad_environment_variables()``.
    '''

    from abjad import ABJCFG

    result = []
    for key in dir(ABJCFG):
        if key.isupper() and not key.startswith('_'):
            result.append((key, getattr(ABJCFG, key)))
    return tuple(result)



from abjad.tools import mathtools


def partition_sequence_by_ratio_of_lengths(sequence, lengths):
    '''.. versionadded:: 2.0

    Partition `sequence` by ratio of `lengths`::

        >>> sequence = tuple(range(10))

    ::

        >>> sequencetools.partition_sequence_by_ratio_of_lengths(sequence, [1, 1, 2])
        [(0, 1, 2), (3, 4), (5, 6, 7, 8, 9)]

    Use rounding magic to avoid fractional part lengths.

    Return list of `sequence` objects.
    '''
    from abjad.tools import sequencetools

    lengths = mathtools.partition_integer_by_ratio(len(sequence), lengths)
    return sequencetools.partition_sequence_by_counts(sequence, lengths, cyclic=False, overhang=False)



from abjad import *


def test_TieChain_leaves_group_by_immediate_parents_01():

    staff = Staff(2 * Measure((2, 8), "c'8 c'8"))
    tietools.TieSpanner(staff.leaves)

    r'''
    \new Staff {
            \time 2/8
            c'8 ~
            c'8 ~
            \time 2/8
            c'8 ~
            c'8
    }
    '''

    groups = tietools.get_tie_chain(staff.leaves[0]).leaves_grouped_by_immediate_parents

    assert len(groups) == 2
    assert groups[0] == list(staff.leaves[:2])
    assert groups[1] == list(staff.leaves[2:])


def test_TieChain_leaves_group_by_immediate_parents_02():

    staff = Staff("c'8 ~ c'8 ~ c'8 ~ c'8")
    groups = tietools.get_tie_chain(staff[0]).leaves_grouped_by_immediate_parents

    assert len(groups) == 1
    assert groups[0] == list(staff.leaves)


def test_TieChain_leaves_group_by_immediate_parents_03():

    note = Note("c'4")
    groups = tietools.get_tie_chain(note).leaves_grouped_by_immediate_parents

    assert len(groups) == 1
    assert groups[0] == [note]



# TODO: optimize without multiple full-component traversal.
def iterate_vertical_moments_in_expr(expr, reverse=False):
    r'''.. versionadded:: 2.10

    Iterate vertical moments forward in `expr`::

        >>> score = Score([])
        >>> staff = Staff(r"\times 4/3 { d''8 c''8 b'8 }")
        >>> score.append(staff)

    ::

        >>> piano_staff = scoretools.PianoStaff([])
        >>> piano_staff.append(Staff("a'4 g'4"))
        >>> piano_staff.append(Staff(r"""\clef "bass" f'8 e'8 d'8 c'8"""))
        >>> score.append(piano_staff)

    ::

        >>> f(score)
        \new Score <<
            \new Staff {
                \fraction \times 4/3 {
                    d''8
                    c''8
                    b'8
                }
            }
            \new PianoStaff <<
                \new Staff {
                    a'4
                    g'4
                }
                \new Staff {
                    \clef "bass"
                    f'8
                    e'8
                    d'8
                    c'8
                }
            >>
        >>

    ::

        >>> for x in verticalitytools.iterate_vertical_moments_in_expr(score):
        ...     x.leaves
        ...
        (Note("d''8"), Note("a'4"), Note("f'8"))
        (Note("d''8"), Note("a'4"), Note("e'8"))
        (Note("c''8"), Note("a'4"), Note("e'8"))
        (Note("c''8"), Note("g'4"), Note("d'8"))
        (Note("b'8"), Note("g'4"), Note("d'8"))
        (Note("b'8"), Note("g'4"), Note("c'8"))

    ::

        >>> for x in verticalitytools.iterate_vertical_moments_in_expr(piano_staff):
        ...     x.leaves
        ...
        (Note("a'4"), Note("f'8"))
        (Note("a'4"), Note("e'8"))
        (Note("g'4"), Note("d'8"))
        (Note("g'4"), Note("c'8"))

    Iterate vertical moments backward in `expr`::

    ::
        
        >>> for x in verticalitytools.iterate_vertical_moments_in_expr(score, reverse=True):
        ...     x.leaves
        ...
        (Note("b'8"), Note("g'4"), Note("c'8"))
        (Note("b'8"), Note("g'4"), Note("d'8"))
        (Note("c''8"), Note("g'4"), Note("d'8"))
        (Note("c''8"), Note("a'4"), Note("e'8"))
        (Note("d''8"), Note("a'4"), Note("e'8"))
        (Note("d''8"), Note("a'4"), Note("f'8"))

    ::

        >>> for x in verticalitytools.iterate_vertical_moments_in_expr(piano_staff, reverse=True):
        ...     x.leaves
        ...
        (Note("g'4"), Note("c'8"))
        (Note("g'4"), Note("d'8"))
        (Note("a'4"), Note("e'8"))
        (Note("a'4"), Note("f'8"))

    Return generator.
    '''
    from abjad.tools import iterationtools
    from abjad.tools import verticalitytools
    from abjad.tools.verticalitytools._yield_vertical_moments_forward_in_expr import \
        _yield_vertical_moments_forward_in_expr

    # OPTIMIZED DROP-IN REPLACEMENT
    if not reverse:
        for x in _yield_vertical_moments_forward_in_expr(expr):
            yield x
    else:
        moments_in_governor = []
        for component in iterationtools.iterate_components_in_expr(expr):
            offset = component.start_offset
            if offset not in moments_in_governor:
                moments_in_governor.append(offset)
        moments_in_governor.sort()
        for moment_in_governor in reversed(moments_in_governor):
            yield verticalitytools.get_vertical_moment_at_offset_in_expr(expr, moment_in_governor)



from abjad.tools import contexttools
from abjad.tools import quantizationtools
import py


def test_BeatwiseQSchema___call___01():

    schema = quantizationtools.BeatwiseQSchema()

    target = schema(5000)




from abjad import *


def test_tuplettools_make_tuplet_from_duration_and_ratio_01():

    duration = Fraction(3, 16)

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1], avoid_dots=True, is_diminution=False)
    assert t.lilypond_format == "\\fraction \\times 3/2 {\n\tc'8\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 1], avoid_dots=True, is_diminution=False)
    assert t.lilypond_format == "\\fraction \\times 3/2 {\n\tc'16\n\tc'16\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 1, 1], avoid_dots=True, is_diminution=False)
    assert t.lilypond_format == "{\n\tc'16\n\tc'16\n\tc'16\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 1, 1, 1], avoid_dots=True, is_diminution=False)
    assert t.lilypond_format == "\\fraction \\times 3/2 {\n\tc'32\n\tc'32\n\tc'32\n\tc'32\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 1, 1, 1, 1], avoid_dots=True, is_diminution=False)
    assert t.lilypond_format == "\\fraction \\times 6/5 {\n\tc'32\n\tc'32\n\tc'32\n\tc'32\n\tc'32\n}"


def test_tuplettools_make_tuplet_from_duration_and_ratio_02():

    duration = Fraction(3, 16)

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1], avoid_dots=True, is_diminution=False)
    assert t.lilypond_format == "\\fraction \\times 3/2 {\n\tc'8\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 2], avoid_dots=True, is_diminution=False)
    assert t.lilypond_format == "{\n\tc'16\n\tc'8\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 2, 2], avoid_dots=True, is_diminution=False)
    assert t.lilypond_format == "\\fraction \\times 6/5 {\n\tc'32\n\tc'16\n\tc'16\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 2, 2, 3], avoid_dots=True, is_diminution=False)
    assert t.lilypond_format == "\\fraction \\times 3/2 {\n\tc'64\n\tc'32\n\tc'32\n\tc'32.\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 2, 2, 3, 3], avoid_dots=True, is_diminution=False)
    assert t.lilypond_format == "\\fraction \\times 12/11 {\n\tc'64\n\tc'32\n\tc'32\n\tc'32.\n\tc'32.\n}"


def test_tuplettools_make_tuplet_from_duration_and_ratio_03():
    '''Interpret negative proportions as rests.
    '''

    duration = Fraction(3, 16)

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, -2, -2, 3, 3], avoid_dots=True, is_diminution=False)
    assert t.lilypond_format == "\\fraction \\times 12/11 {\n\tc'64\n\tr32\n\tr32\n\tc'32.\n\tc'32.\n}"


def test_tuplettools_make_tuplet_from_duration_and_ratio_04():
    '''Reduce proportions relative to each other.
    '''

    duration = Fraction(3, 16)

    t1 = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, -2, -2, 3, 3], avoid_dots=True, is_diminution=False)
    t2 = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [2, -4, -4, 6, 6], avoid_dots=True, is_diminution=False)
    assert t1.lilypond_format == t2.lilypond_format

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        Fraction(1, 8), [27], avoid_dots=True, is_diminution=False)
    assert t.lilypond_format == "{\n\tc'8\n}"


def test_tuplettools_make_tuplet_from_duration_and_ratio_05():

    duration = Fraction(3, 16)

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1], avoid_dots=False, is_diminution=False)
    assert t.lilypond_format == "{\n\tc'8.\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 1], avoid_dots=False, is_diminution=False)
    assert t.lilypond_format == "{\n\tc'16.\n\tc'16.\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 1, 1], avoid_dots=False, is_diminution=False)
    assert t.lilypond_format == "{\n\tc'16\n\tc'16\n\tc'16\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 1, 1, 1], avoid_dots=False, is_diminution=False)
    assert t.lilypond_format == "{\n\tc'32.\n\tc'32.\n\tc'32.\n\tc'32.\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 1, 1, 1, 1], avoid_dots=False, is_diminution=False)
    assert t.lilypond_format == "\\fraction \\times 8/5 {\n\tc'64.\n\tc'64.\n\tc'64.\n\tc'64.\n\tc'64.\n}"


def test_tuplettools_make_tuplet_from_duration_and_ratio_06():

    duration = Fraction(3, 16)

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1], avoid_dots=False, is_diminution=False)
    assert t.lilypond_format == "{\n\tc'8.\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 2], avoid_dots=False, is_diminution=False)
    assert t.lilypond_format == "{\n\tc'16\n\tc'8\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 2, 2], avoid_dots=False, is_diminution=False)
    assert t.lilypond_format == "\\fraction \\times 8/5 {\n\tc'64.\n\tc'32.\n\tc'32.\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 2, 2, 3], avoid_dots=False, is_diminution=False)
    assert t.lilypond_format == "\\fraction \\times 3/2 {\n\tc'64\n\tc'32\n\tc'32\n\tc'32.\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 2, 2, 3, 3], avoid_dots=False, is_diminution=False)
    assert t.lilypond_format == "\\fraction \\times 12/11 {\n\tc'64\n\tc'32\n\tc'32\n\tc'32.\n\tc'32.\n}"


def test_tuplettools_make_tuplet_from_duration_and_ratio_07():
    '''Reduce proportions relative to each other.
    '''

    duration = Fraction(3, 16)

    t1 = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 2, 2, 3, 3], avoid_dots=False, is_diminution=False)
    t2 = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [2, 4, 4, 6, 6], avoid_dots=False, is_diminution=False)
    assert t1.lilypond_format == t2.lilypond_format

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        Fraction(1, 8), [27], avoid_dots=False, is_diminution=False)
    assert t.lilypond_format == "{\n\tc'8\n}"


def test_tuplettools_make_tuplet_from_duration_and_ratio_08():

    duration = Fraction(3, 16)

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1], avoid_dots=True, is_diminution=True)
    assert t.lilypond_format == "\\fraction \\times 3/4 {\n\tc'4\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 1], avoid_dots=True, is_diminution=True)
    assert t.lilypond_format == "\\fraction \\times 3/4 {\n\tc'8\n\tc'8\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 1, 1], avoid_dots=True, is_diminution=True)
    assert t.lilypond_format == "{\n\tc'16\n\tc'16\n\tc'16\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 1, 1, 1], avoid_dots=True, is_diminution=True)
    assert t.lilypond_format == "\\fraction \\times 3/4 {\n\tc'16\n\tc'16\n\tc'16\n\tc'16\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 1, 1, 1, 1], avoid_dots=True, is_diminution=True)
    assert t.lilypond_format == "\\fraction \\times 3/5 {\n\tc'16\n\tc'16\n\tc'16\n\tc'16\n\tc'16\n}"


def test_tuplettools_make_tuplet_from_duration_and_ratio_09():

    duration = Fraction(3, 16)

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1], avoid_dots=True, is_diminution=True)
    assert t.lilypond_format == "\\fraction \\times 3/4 {\n\tc'4\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 2], avoid_dots=True, is_diminution=True)
    assert t.lilypond_format == "{\n\tc'16\n\tc'8\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 2, 2], avoid_dots=True, is_diminution=True)
    assert t.lilypond_format == "\\fraction \\times 3/5 {\n\tc'16\n\tc'8\n\tc'8\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 2, 2, 3], avoid_dots=True, is_diminution=True)
    assert t.lilypond_format == "\\fraction \\times 3/4 {\n\tc'32\n\tc'16\n\tc'16\n\tc'16.\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 2, 2, 3, 3], avoid_dots=True, is_diminution=True)
    assert t.lilypond_format == "\\fraction \\times 6/11 {\n\tc'32\n\tc'16\n\tc'16\n\tc'16.\n\tc'16.\n}"


def test_tuplettools_make_tuplet_from_duration_and_ratio_10():
    '''Interpret negative proportions as rests.
    '''

    duration = Fraction(3, 16)

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, -2, -2, 3, 3], avoid_dots=True, is_diminution=True)
    assert t.lilypond_format == "\\fraction \\times 6/11 {\n\tc'32\n\tr16\n\tr16\n\tc'16.\n\tc'16.\n}"


def test_tuplettools_make_tuplet_from_duration_and_ratio_11():
    '''Reduce propotions relative to each other.
    '''

    duration = Fraction(3, 16)

    t1 = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, -2, -2, 3, 3], avoid_dots=True, is_diminution=True)
    t2 = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [2, -4, -4, 6, 6], avoid_dots=True, is_diminution=True)
    assert t1.lilypond_format == t2.lilypond_format

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        Fraction(1, 8), [27])
    assert t.lilypond_format == "{\n\tc'8\n}"


def test_tuplettools_make_tuplet_from_duration_and_ratio_12():

    duration = Fraction(3, 16)

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1], avoid_dots=False, is_diminution=True)
    assert t.lilypond_format == "{\n\tc'8.\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 1], avoid_dots=False, is_diminution=True)
    assert t.lilypond_format == "{\n\tc'16.\n\tc'16.\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 1, 1], avoid_dots=False, is_diminution=True)
    assert t.lilypond_format == "{\n\tc'16\n\tc'16\n\tc'16\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 1, 1, 1], avoid_dots=False, is_diminution=True)
    assert t.lilypond_format == "{\n\tc'32.\n\tc'32.\n\tc'32.\n\tc'32.\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 1, 1, 1, 1], avoid_dots=False, is_diminution=True)
    assert t.lilypond_format == "\\times 4/5 {\n\tc'32.\n\tc'32.\n\tc'32.\n\tc'32.\n\tc'32.\n}"


def test_tuplettools_make_tuplet_from_duration_and_ratio_13():

    duration = Fraction(3, 16)

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1], avoid_dots=False, is_diminution=True)
    assert t.lilypond_format == "{\n\tc'8.\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 2], avoid_dots=False, is_diminution=True)
    assert t.lilypond_format == "{\n\tc'16\n\tc'8\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 2, 2], avoid_dots=False, is_diminution=True)
    assert t.lilypond_format == "\\times 4/5 {\n\tc'32.\n\tc'16.\n\tc'16.\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 2, 2, 3], avoid_dots=False, is_diminution=True)
    assert t.lilypond_format == "\\fraction \\times 3/4 {\n\tc'32\n\tc'16\n\tc'16\n\tc'16.\n}"

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, 2, 2, 3, 3], avoid_dots=False, is_diminution=True)
    assert t.lilypond_format == "\\fraction \\times 6/11 {\n\tc'32\n\tc'16\n\tc'16\n\tc'16.\n\tc'16.\n}"


def test_tuplettools_make_tuplet_from_duration_and_ratio_14():
    '''Reduce proportions relative to each other.
    '''

    duration = Fraction(3, 16)

    t1 = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [1, -2, -2, 3, 3], avoid_dots=False, is_diminution=True)
    t2 = tuplettools.make_tuplet_from_duration_and_ratio(
        duration, [2, -4, -4, 6, 6], avoid_dots=False, is_diminution=True)
    assert t1.lilypond_format == t2.lilypond_format

    t = tuplettools.make_tuplet_from_duration_and_ratio(
        Fraction(1, 8), [27], avoid_dots=False, is_diminution=True)
    assert t.lilypond_format == "{\n\tc'8\n}"


def test_tuplettools_make_tuplet_from_duration_and_ratio_15():
    '''Coerce duration.
    '''
    
    tuplet = tuplettools.make_tuplet_from_duration_and_ratio(
        (1, 4), [1, -1, 1], avoid_dots=True, is_diminution=True)

    assert tuplet.lilypond_format == "\\times 2/3 {\n\tc'8\n\tr8\n\tc'8\n}"



import itertools


def yield_topmost_components_grouped_by_type(expr):
    r'''.. versionadded:: 2.0

    Example 1. Yield topmost components in `expr` grouped by type::

        >>> staff = Staff(r"\times 2/3 { c'8 d'8 r8 } \times 2/3 { r8 <e' g'>8 <f' a'>8 }")
        >>> staff.extend("g'8 a'8 r8 r8 <b' d''>8 <c'' e''>8")

    ::

        >>> f(staff)
        \new Staff {
            \times 2/3 {
                c'8
                d'8
                r8
            }
            \times 2/3 {
                r8
                <e' g'>8
                <f' a'>8
            }
            g'8
            a'8
            r8
            r8
            <b' d''>8
            <c'' e''>8
        }

    ::

        >>> for x in componenttools.yield_topmost_components_grouped_by_type(staff):
        ...     x
        (Tuplet(2/3, [c'8, d'8, r8]), Tuplet(2/3, [r8, <e' g'>8, <f' a'>8]))
        (Note("g'8"), Note("a'8"))
        (Rest('r8'), Rest('r8'))
        (Chord("<b' d''>8"), Chord("<c'' e''>8"))

    Example 2. Yield leaves at all score levels in `expr` grouped by type::

        >>> leaves = iterationtools.iterate_leaves_in_expr(staff)

    ::

        >>> for x in componenttools.yield_topmost_components_grouped_by_type(leaves):
        ...     x
        (Note("c'8"), Note("d'8"))
        (Rest('r8'), Rest('r8'))
        (Chord("<e' g'>8"), Chord("<f' a'>8"))
        (Note("g'8"), Note("a'8"))
        (Rest('r8'), Rest('r8'))
        (Chord("<b' d''>8"), Chord("<c'' e''>8"))

    Return generator.
    '''

    grouper = itertools.groupby(expr, type)
    for leaf_type, generator in grouper:
        yield tuple(generator)
